SystemOrganization addCategory: #'Zinc-Tests'!

TestCase subclass: #ZnBivalentWriteStreamTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnBivalentWriteStreamTests methodsFor: 'testing' stamp: ''!
testByteWriting
	| string |
	string := String streamContents: [ :stream | | writeStream |
		writeStream := ZnBivalentWriteStream on: stream.
		writeStream nextPut: 97; nextPutAll: #(98 99) asByteArray ].
	self assert: string asByteArray = #(97 98 99) asByteArray! !

!ZnBivalentWriteStreamTests methodsFor: 'testing' stamp: ''!
testCharacterWriting
	| bytes |
	bytes := ByteArray streamContents: [ :stream | | writeStream |
		writeStream := ZnBivalentWriteStream on: stream.
		writeStream nextPut: $a; space; nextPutAll: '123' ].
	self assert: bytes = 'a 123' asByteArray! !

TestCase subclass: #ZnChunkedStreamTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnChunkedStreamTests methodsFor: 'private' stamp: ''!
nextPutAll: collection on: stream chunked: size
	| written |
	written := 0.
	[ written < collection size ] whileTrue: [ | chunkSize |
		chunkSize := size min: (collection size - written).
		chunkSize printOn: stream base: 16.
		stream nextPutAll: String crlf.
		chunkSize timesRepeat: [ stream nextPut: (collection at: (written := written + 1)) ].
		stream nextPutAll: String crlf ].
	stream print: 0; nextPutAll: String crlf; nextPutAll: String crlf! !

!ZnChunkedStreamTests methodsFor: 'testing' stamp: ''!
testBinaryWriteRead
	| data written read |
	data := ByteArray new: 1024 streamContents: [ :out |
		1024 timesRepeat: [ out nextPut: 256 atRandom - 1 ] ].
	written := ByteArray streamContents: [ :out |
		| stream |
		stream := ZnBufferedWriteStream on: (ZnChunkedWriteStream on: out).
		stream sizeBuffer: 256.
		1 to: 1024 by: 256 do: [ :each |
			stream next: 256 putAll: data startingAt: each ].
		stream close ].
	read := (ZnChunkedReadStream on: written readStream) upToEnd.
	self assert: read equals: data! !

!ZnChunkedStreamTests methodsFor: 'testing' stamp: ''!
testGzipWriteRead
	| data written read |
	data := String streamContents: [ :out |
		{ Collection. Stream. Exception. Magnitude } do: [ :cls |
			cls allSubclassesDo: [ :each | out nextPutAll: each comment ] ] ].
	data := data asByteArray.
	written := ByteArray streamContents: [ :out |
		| stream chunkedStream |
		stream := GZipWriteStream on: (chunkedStream := ZnChunkedWriteStream on: out).
		stream nextPutAll: data.
		stream finish.
		chunkedStream finish ].
	read := (GZipReadStream on: (ZnChunkedReadStream on: written readStream)) upToEnd.
	self assert: read equals: data! !

!ZnChunkedStreamTests methodsFor: 'testing' stamp: ''!
testReading
	| data chunked plain |
	data := String withAll: ($a to: $z), ($A to: $Z).
	chunked := String streamContents: [ :stream |
		self nextPutAll: data on: stream chunked: 16 ].
	plain := (ZnChunkedReadStream on: chunked readStream) upToEnd.
	self assert: plain equals: data! !

!ZnChunkedStreamTests methodsFor: 'testing' stamp: ''!
testReadingBuffered
	| data chunked plain buffer readStream |
	data := String withAll: ($a to: $z), ($A to: $Z).
	chunked := String streamContents: [ :stream |
		self nextPutAll: data on: stream chunked: 16 ].
	readStream := ZnChunkedReadStream on: chunked readStream.
	buffer := String new: 11.
	plain := String streamContents: [ :output | | readCount |
		[ readStream atEnd ] whileFalse: [
			readCount := readStream readInto: buffer startingAt: 1 count: buffer size.
			output next: readCount putAll: buffer ] ].
	self assert: plain equals: data! !

!ZnChunkedStreamTests methodsFor: 'testing' stamp: ''!
testReadingGzip
	| data gzipped chunked plain |
	data := String withAll: ($a to: $z), ($A to: $Z).
	gzipped := String streamContents: [ :stream |
		(GZipWriteStream on: stream) nextPutAll: data; close ].
	chunked := String streamContents: [ :stream |
		self nextPutAll: gzipped on: stream chunked: 16 ].
	plain := (GZipReadStream on: (ZnChunkedReadStream on: chunked readStream)) upToEnd.
	self assert: data equals: plain ! !

!ZnChunkedStreamTests methodsFor: 'testing' stamp: ''!
testReadingGzipStepped
	| data gzipped chunked unchunked ungzipped |
	data := String withAll: ($a to: $z), ($A to: $Z).
	gzipped := String streamContents: [ :stream |
		(GZipWriteStream on: stream) nextPutAll: data; close ].
	chunked := String streamContents: [ :stream |
		self nextPutAll: gzipped on: stream chunked: 16 ].
	unchunked := (ZnChunkedReadStream on: chunked readStream) upToEnd.
	ungzipped := (GZipReadStream on: unchunked) upToEnd.
	self assert: gzipped equals: unchunked.
	self assert: data equals: ungzipped ! !

!ZnChunkedStreamTests methodsFor: 'testing' stamp: ''!
testReadingNext
	| data chunked plain |
	data := String withAll: ($a to: $z), ($A to: $Z).
	chunked := String streamContents: [ :stream |
		self nextPutAll: data on: stream chunked: 16 ].
	plain := (ZnChunkedReadStream on: chunked readStream) next: 26 + 26.
	self assert: plain equals: data! !

!ZnChunkedStreamTests methodsFor: 'testing' stamp: ''!
testReadingNextInto
	| data chunked plain |
	data := String withAll: ($a to: $z), ($A to: $Z).
	chunked := String streamContents: [ :stream |
		self nextPutAll: data on: stream chunked: 16 ].
	plain := String new: 26 + 26.
	(ZnChunkedReadStream on: chunked readStream) next: 26 + 26 into: plain.
	self assert: plain equals: data! !

!ZnChunkedStreamTests methodsFor: 'testing' stamp: ''!
testWriting
	| chunks result reference |
	chunks := #( 'ABC' 'DEF' 'GHI' 'JKL' ).
	result := String streamContents: [ :out |
		| stream |
		stream := ZnChunkedWriteStream on: out.
		chunks do: [ :each | stream nextPutAll: each ].
		stream finish ].
	reference := String streamContents: [ :out |
		self nextPutAll: (String new join: chunks) on: out chunked: 3 ].
	self assert: result equals: reference! !

!ZnChunkedStreamTests methodsFor: 'testing' stamp: ''!
testWritingBuffered
	| data written read |
	data := String withAll: ($a to: $z), ($A to: $Z).
	written := String streamContents: [ :out |
		| stream |
		stream := ZnBufferedWriteStream on: (ZnChunkedWriteStream on: out).
		stream sizeBuffer: 16.
		stream nextPutAll: data.
		stream close ].
	read := (ZnChunkedReadStream on: written readStream) upToEnd.
	self assert: read equals: data! !

TestCase subclass: #ZnClientTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnClientTests methodsFor: 'accessing' stamp: ''!
smallHtmlUrl
	^ 'http://zn.stfx.eu/zn/small.html' asZnUrl! !

!ZnClientTests methodsFor: 'accessing' stamp: ''!
t3EasyGeoIPUrl
	^ 'http://easy.t3-platform.net/rest/geo-ip' asZnUrl! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testConnectionCloseOnReuseTimeout
	| client firstSocket secondSocket|
	client := ZnClient new.
	client connectionReuseTimeout: 0.
	self withServerDo: [ :server | 
		client 
			url: server localUrl;
			get.
		firstSocket := client connection.
		client get.
		secondSocket := client connection.		
		self assert: (firstSocket = secondSocket) not.
		self deny: firstSocket isConnected.
		firstSocket close.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testConstruction
	| client |
	(client := ZnClient new)
		http;
		host: 'www.example.com';
		port: 8080;
		addPath: 'foo';
		addPath: 'a space';
		addPath: 'file.txt';
		queryAt: 'key' put: '123456';
		headerAt: 'X-token' put: 'ABCDEF'.
	self assert: client request url host = 'www.example.com'.
	self assert: client request url port = 8080.
	self assert: client request url pathQueryFragmentPrintString = '/foo/a%20space/file.txt?key=123456'.
	self assert: (client request headers at: 'X-token') = 'ABCDEF'! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testCookies
	| client |
	self withServerDo: [ :server |
		server onRequestRespond: [ :request | | cookie1 cookie2 response |
			cookie1 := request cookies detect: [ :each | each name = 'x' ] ifNone: [ ZnCookie name: 'x' value: '0' ].
			cookie2 := request cookies detect: [ :each | each name = 'y' ] ifNone: [ ZnCookie name: 'y' value: '100' ].
			cookie1 value: (cookie1 value asInteger + 1) asString.
			cookie2 value: (cookie2 value asInteger + 2) asString.
			response := ZnResponse ok: (ZnEntity text: 'OK').
			response addCookie: cookie1; addCookie: cookie2.
			response ].
		client := ZnClient new url: server localUrl; yourself.
		self assert: (client session cookieJar cookieAt: 'x' forUrl: client request url) isNil.
		self assert: (client session cookieJar cookieAt: 'y' forUrl: client request url) isNil.
		client get.
		self assert: client isSuccess.
		self assert: (client session cookieJar cookieAt: 'x' forUrl: client request url) value = '1'.
		self assert: (client session cookieJar cookieAt: 'y' forUrl: client request url) value = '102'.
		client get.
		self assert: client isSuccess.
		self assert: (client session cookieJar cookieAt: 'x' forUrl: client request url) value = '2'.
		self assert: (client session cookieJar cookieAt: 'y' forUrl: client request url) value = '104'.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/1/2018 23:49'!
testDownloadSmallHTML
	| client |
	ZnFileSystemUtils deleteIfExists: self smallHtmlUrl pathSegments last.
	"First download to a directory"
	(client := ZnClient new)
		url: self smallHtmlUrl;
		downloadTo: ZnFileSystemUtils defaultDirectoryPath.
	self assert: client isSuccess.
	self assert: client response contentType = ZnMimeType textHtml.
	ZnFileSystemUtils 
		readStreamFor: self smallHtmlUrl pathSegments last
		do: [ :stream | self assert: (stream upToEnd includesSubstring: 'Small') ].
	ZnFileSystemUtils deleteIfExists: self smallHtmlUrl pathSegments last.
	"Second download to an explicitly named file"
	client 
		url: self smallHtmlUrl;
		downloadTo: self smallHtmlUrl pathSegments last.
	self assert: client isSuccess.
	self assert: client response contentType = ZnMimeType textHtml.
	ZnFileSystemUtils 
		readStreamFor: self smallHtmlUrl pathSegments last
		do: [ :stream | self assert: (stream upToEnd includesSubstring: 'Small') ].	
	client close.
	ZnFileSystemUtils deleteIfExists: self smallHtmlUrl pathSegments last
! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetAfterPost
	self withServerDo: [ :server | | client |
		server onRequestRespond: [ :request | 
			request uri firstPathSegment = 'one'
				ifTrue: [
					(request method = #POST and: [ request hasEntity ])
						ifTrue: [ ZnResponse ok: (ZnEntity text: 'OK for one') ]
						ifFalse: [ ZnResponse badRequest: request ] ]
				ifFalse: [
					request uri firstPathSegment = 'two'
						ifTrue: [ 
							(request method = #GET and: [ request hasEntity not ])
								ifTrue: [ ZnResponse ok: (ZnEntity text: 'OK for two') ]
								ifFalse: [ ZnResponse badRequest: request ] ]
		 				ifFalse: [ ZnResponse notFound: request uri ] ] ].
		(client := ZnClient new) 
			autoResetEntityMethods: #(HEAD DELETE GET);
			url: server localUrl;
			url: 'one';
			entity: (ZnEntity text: 'One two three');
			post.
		self assert: client isSuccess.
		self assert: client contents = 'OK for one'.
		client
			url: 'two';
			get.
		self assert: client isSuccess.
		self assert: client contents = 'OK for two'.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetForm
	| client |
	self withServerDo: [ :server |
		(client := ZnClient new)
			url: server localUrl;
			addPath: 'echo';
			formAt: 'username' put: 'john';
			formAdd: 'password' -> 'secret';
			get.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'john').
		self assert: (client contents includesSubstring: 'secret') ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetGeoIP
	| result client |
	result := (client := ZnClient new)
		systemPolicy;
		url: self t3EasyGeoIPUrl;
		queryAt: 'address' put: '81.83.7.35';
		accept: ZnMimeType applicationJson;
		contentReader: [ :entity | 
					Smalltalk globals
						at: #NeoJSONReader
						ifPresent: [ :parserClass | parserClass fromString: entity contents ]
						ifAbsent: [ ^ self ] ];
		ifFail: [ ^ self fail ];
		get.
	self assert: result isDictionary.
	self assert: (result at: #country) = 'BE'.
	client close! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetLargeWideStringHTML
	| client text |
	text := '<html>{1}<h1>Czech in Czech is {2}e{3}tina.</h1>{1}</html>' 
		format: { 
			String new: ZnUtils streamingBufferSize withAll: $X.
			269 asCharacter. 
			353 asCharacter }.
	self assert: text isWideString.
	self withServerDo: [ :server |
		server onRequestRespond: [ :request | ZnResponse ok: (ZnEntity html: text) ].
		(client := ZnClient new)
			get: server localUrl.
		self assert: client isSuccess.
		self assert: client response contents equals: text.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetSmallHTML
	| client |
	(client := ZnClient new)
		url: self smallHtmlUrl;
		get.
	self assert: client isSuccess.
	self assert: client isContentTypeAcceptable.
	self assert: client response contentType = ZnMimeType textHtml.
	self assert: (client contents includesSubstring: 'Small').
	self assert: client isConnected.
	client close.
	self deny: client isConnected! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetSmallHTMLBinary
	| client html bytes |
	
	(client := ZnClient new)
		url: self smallHtmlUrl;
		get.
	self assert: client isSuccess.
	self assert: client response contentType equals: ZnMimeType textHtml.
	self assert: (client contents includesSubstring: 'Small').
	html := client contents.
	self assert: html isString.
	
	client beBinary; get.
	self assert: client isSuccess.
	self assert: client response contentType = ZnMimeType textHtml.
	bytes := client contents.
	self deny: bytes isString.
	self assert: bytes utf8Decoded equals: html.
	
	client close! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetSmallHTMLOneShot
	| client |
	(client := ZnClient new)
		beOneShot;
		url: self smallHtmlUrl;
		get.
	self assert: client isSuccess.
	self assert: client response contentType = ZnMimeType textHtml.
	self assert: (client contents includesSubstring: 'Small').
	self deny: client isConnected! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetSmallHTMLStreaming
	| client result contents |
	result := (client := ZnClient new)
		url: self smallHtmlUrl;
		streaming: true;
		get.
	self assert: client isSuccess.
	self assert: client response contentType = ZnMimeType textHtml.
	self assert: result isStream.
	self assert: client entity stream = result.
	contents := ZnUTF8Encoder new decodeBytes: result upToEnd.
	self assert: (contents includesSubstring: 'Small').
	client close! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetSmallHTMLTwice
	| client |
	(client := ZnClient new)
		url: self smallHtmlUrl;
		get.
	self assert: client isSuccess.
	self assert: client response contentType = ZnMimeType textHtml.
	self assert: (client contents includesSubstring: 'Small').
	client 
		url: self smallHtmlUrl pathPrintString;
		get.
	self assert: client isSuccess.
	self assert: client response contentType = ZnMimeType textHtml.
	self assert: (client contents includesSubstring: 'Small').
	client close! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetSmallHTMLUrlConstruction
	| client |
	(client := ZnClient new)
		http;
		host: self smallHtmlUrl host;
		path: self smallHtmlUrl pathPrintString;
		get.
	self assert: client isSuccess.
	self assert: client response contentType = ZnMimeType textHtml.
	self assert: (client contents includesSubstring: 'Small').
	client close! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testGetWideStringHTML
	| client text |
	text := '<html><h1>Czech is in Czech {1}e{2}tina.</h1></html>' format: { 269 asCharacter. 353 asCharacter}.
	self assert: text isWideString.
	self withServerDo: [ :server |
		server onRequestRespond: [ :request | ZnResponse ok: (ZnEntity html: text) ].
		(client := ZnClient new)
			get: server localUrl.
		self assert: client isSuccess.
		self assert: client response contents equals: text.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testHeadSmallHTML
	| client |
	self withServerDo: [ :server |
		(client := ZnClient new)
			url: server localUrl;
			addPath: 'echo';
			head.
		self assert: client isSuccess.
		self assert: client response contentLength > 0.
		self assert: client response hasEntity not.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testIfFailNonExistingHost
	| client |
	"Note that when there is an HTTP proxy set, there will be no direct Error"
	(client := ZnClient new)
		beOneShot;
		url: 'http://unknown-stfx.eu';
		ifFail: [ : exception |
			^ self assert: (exception isKindOf: NetworkError) | (exception isKindOf: ZnHttpUnsuccessful) ];
		get.
	self assert: client isSuccess not.
	client
		enforceHttpSuccess: true;
		get.
	self fail! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testIfFailNotFound
	| client |
	(client := ZnClient new)
		beOneShot;
		url: 'http://stfx.eu/non-existing';
		ifFail: [ :exception |
			^ self assert: (exception isKindOf: ZnHttpUnsuccessful) ];
		get.
	self assert: client isSuccess not.
	client
		enforceHttpSuccess: true;
		get.
	self fail! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testIfFailWrongType
	| client |
	(client := ZnClient new)
		beOneShot;
		accept: ZnMimeType imagePng;
		enforceAcceptContentType: true;
		url: self smallHtmlUrl;
		ifFail: [ :exception |
			^ self assert: (exception isKindOf: ZnUnexpectedContentType) ];
		get.
	self fail! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testIfModifiedSinceNotModified
	| client  response |
	(client := ZnClient new) host: 'stfx.eu'.
	[
		response := client 
			url: '/small.html'; 
			setIfModifiedSince: (Date year: 2012 month: 9 day: 1);
			get; 
			response.
		self assert: response isNotModified.
		self assert: response hasEntity not ] ensure: [ client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testLogging
	| client logEventCollection count |
	logEventCollection := OrderedCollection new.
	ZnLogEvent announcer 
		when: ZnLogEvent do: [ :event | logEventCollection add: event ]. 
	(client := ZnClient new)
		clientId: #C1;
		beOneShot.
	client get: self smallHtmlUrl.
	client close.
	count := ZnLogEvent announcer numberOfSubscriptions.
	ZnLogEvent announcer 
		unsubscribe: self.
	self assert: ZnLogEvent announcer numberOfSubscriptions equals: count -1.
	self deny: logEventCollection isEmpty.
	self assert: logEventCollection anyOne clientId equals: #C1! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testOptions
	| client |
	self withServerDo: [ :server |
		server onRequestRespond: [ :request | 
			request method = #OPTIONS
				ifTrue: [ | response |
					response := ZnResponse noContent.
					response headers at: #Allow put: 'GET, HEAD'.
					response ]
				ifFalse: [ ZnResponse badRequest: request ] ].
		(client := ZnClient new)
			options: server localUrl.
		self assert: client isSuccess.
		self deny: client response hasEntity.
		self assert: (client response headers at: #Allow) equals: 'GET, HEAD'.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testPatch
	| client |
	self withServerDo: [ :server |
		server onRequestRespond: [ :request | 
			request method = #PATCH
				ifTrue: [| response |
					response := ZnResponse noContent.
					response headers at: #Etag put: '"e0023aa4f"'.
					response]
				ifFalse: [ ZnResponse badRequest: request ] ].
		(client := ZnClient new)
			patch: server localUrl contents: 'Some data as text'.
		self assert: client isSuccess.
		self deny: client response hasEntity.
		self assert: (client response headers at: #Etag) equals: '"e0023aa4f"'.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testPostForm
	| client |
	self withServerDo: [ :server |
		(client := ZnClient new)
			url: server localUrl;
			addPath: 'echo';
			formAt: 'username' put: 'john';
			formAdd: 'password' -> 'secret';
			post.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'john').
		self assert: (client contents includesSubstring: 'secret') ]! !

!ZnClientTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/1/2018 23:51'!
testPostMultipart
	| client filename |
	ZnFileSystemUtils deleteIfExists: 'testPostMultiPart.txt'.
	filename := ZnFileSystemUtils fullNameFor: 'testPostMultiPart.txt'.
	ZnFileSystemUtils writeStreamFor: filename do: [ :stream | stream nextPutAll: '0123456789' ].
	self withServerDo: [ :server |
		(client := ZnClient new)
			url: server localUrl;
			addPath: 'echo';
			addPart: (ZnMimePart fieldName: 'username' value: 'john');
			addPart: (ZnMimePart fieldName: 'foo' entity: (ZnEntity text: '123'));
			addPart: (ZnMimePart fieldName: 'file' fileNamed: filename);
			post.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'multipart') ].
	ZnFileSystemUtils deleteIfExists: 'testPostMultiPart.txt'.
! !

!ZnClientTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/1/2018 23:54'!
testPostMultipartBinary
	| client filename |
	ZnFileSystemUtils deleteIfExists: 'testPostMultiPart.bin'.
	filename := ZnFileSystemUtils fullNameFor: 'testPostMultiPart.bin'.
	ZnFileSystemUtils binaryWriteStreamFor: filename do: [ :stream | stream nextPutAll: #[0 1 2 3 4 5 6 7 8 9] ].
	self withServerDo: [ :server |
		(client := ZnClient new)
			url: server localUrl;
			addPath: 'echo';
			addPart: (ZnMimePart fieldName: 'username' value: 'john');
			addPart: (ZnMimePart fieldName: 'foo' entity: (ZnEntity bytes: #[1 2 3]));
			addPart: (ZnMimePart fieldName: 'file' fileNamed: filename);
			post.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'multipart') ].
	ZnFileSystemUtils deleteIfExists: 'testPostMultiPart.bin'.
! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testPostTwice
	| client |
	self withServerDo: [ :server |
		(client := ZnClient new)
			url: server localUrl;
			addPath: 'echo';
			contents: 'Foo bar!!';
			post.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'Foo bar!!').
		client
			contents: 'Another one ?';
			post.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'Another one ?').
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testPrepareRequest
	| client |
	self withServerDo: [ :server |
		(client := ZnClient new)
			url: server localUrl;
			addPath: 'echo';
			prepareRequest: [ :request | request setAuthorization: 'my-signature' ];
			get.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'my-signature').
		client
			url: server localUrl;
			addPath: 'echo';
			get.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'my-signature').
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testProgress
	"[ :bar |
		bar title: 'Downloading Sources...'.
		[
			ZnClient new 
				url: 'http://files.pharo.org/sources/PharoV30.sources'; 
				signalProgress: true;	
				downloadTo: FileLocator temp ]
			on: HTTPProgress 
			do: [ :progress |
				progress isEmpty ifFalse: [ bar current: progress percentage ]. 
				progress resume ] ] asJob run."
		
	self 
		should: [ 
			ZnClient new
				beOneShot;
				signalProgress: true;
				get: self smallHtmlUrl ] 
		raise: HTTPProgress
! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testProgressNoIfFail
	self 
		should: [ 
			ZnClient new
				beOneShot;
				signalProgress: true;
				ifFail: [ self fail ];
				get: self smallHtmlUrl ] 
		raise: HTTPProgress
! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testQueryGoogle
	| client |
	(client := ZnClient new)
		http;
		host: 'www.google.com';
		addPath: 'search';
		queryAt: 'q' put: 'Pharo Smalltalk';
		get.
	self assert: client isSuccess.
	self assert: (client response contentType matches: ZnMimeType textHtml).
	self assert: (client contents includesSubstring: 'pharo.org').
	client close! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testRedirect
	| client response target |
	target := 'http://zn.stfx.eu'.
	client := ZnClient new url: target.
	client get.
	self assert: client isSuccess.
	client
		close;
		maxNumberOfRedirects: 0;
		url: target.
	self should: [ client get ] raise: ZnTooManyRedirects.
	client close.
	response := [ 
		ZnClient new
			beOneShot;
			maxNumberOfRedirects: 0;
			get: target;
			response ] on: ZnTooManyRedirects do: [ :exception | exception resume ].
	self assert: response isRedirect! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testRedirectDontFollow
	| client target |
	target := 'http://zn.stfx.eu'.
	(client := ZnClient new) 
		dontFollowRedirects;
		get: target.
	self assert: client response isRedirect.
	client
		enforceHttpSuccess: true;
		get: target.
	self assert: client response isRedirect.
	client close! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testRedirectWithCookies
	self withServerDo: [ :server | | client cookie |
		server onRequestRespond: [ :request | 
			request uri firstPathSegment = 'one'
				ifTrue: [ 
					(ZnResponse redirect: 'two')
						addCookie: (ZnCookie name: 'session' value: '123456');
						yourself ]
				ifFalse: [
					cookie := request cookies detect: [ :each | each name = 'session' ] ifNone: [ nil ].
					(request uri firstPathSegment = 'two' and: [ cookie notNil and: [ cookie value = '123456' ] ])
						ifTrue: [ ZnResponse ok: (ZnEntity text: 'OK!!') ]
		 				ifFalse: [ ZnResponse badRequest: request ] ] ].
		(client := ZnClient new) 
			url: server localUrl; addPath: 'one'; 
			post.
		self assert: client isSuccess.
		self assert: client contents = 'OK!!'.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testRedirectWithCustomHeader
	self withServerDo: [ :server | | client cookie |
		server onRequestRespond: [ :request | 
			(request headers at: 'X-Custom' ifAbsent: [ nil ]) = 'The Secret'
				ifFalse: [ ZnResponse badRequest: request ]
				ifTrue: [
					request uri firstPathSegment = 'one'
						ifTrue: [ ZnResponse redirect: 'two' ]
						ifFalse: [
							request uri firstPathSegment = 'two'
								ifTrue: [ ZnResponse ok: (ZnEntity text: 'OK!!') ]
		 						ifFalse: [ ZnResponse badRequest: request ] ] ] ].
		(client := ZnClient new) 
			url: server localUrl; 
			addPath: 'one';
			headerAt: 'X-Custom' put: 'The Secret'; 
			get.
		self assert: client isSuccess.
		self assert: client contents = 'OK!!'.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testRelativeRedirect
	self withServerDo: [ :server | | client |
		server onRequestRespond: [ :request | 
			request uri firstPathSegment = 'one'
				ifTrue: [ ZnResponse redirect: 'two' ]
				ifFalse: [
					request uri firstPathSegment = 'two'
						ifTrue: [ ZnResponse ok: (ZnEntity text: 'OK!!') ]
		 				ifFalse: [ ZnResponse notFound: request uri ] ] ].
		(client := ZnClient new) 
			url: server localUrl; addPath: 'one'; 
			post.
		self assert: client isSuccess.
		self assert: client contents = 'OK!!'.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testRelativeRedirect307
	self withServerDo: [ :server | | client |
		server onRequestRespond: [ :request | 
			request uri firstPathSegment = 'one'
				ifTrue: [ (ZnResponse redirect: 'two') statusLine: (ZnStatusLine code: 307); yourself ]
				ifFalse: [
					((request uri firstPathSegment = 'two') 
							and: [ request method = #POST and: [ request contents = 'BODY' ] ])
						ifTrue: [ ZnResponse ok: (ZnEntity text: 'OK!!') ]
		 				ifFalse: [ ZnResponse notFound: request uri ] ] ].
		(client := ZnClient new) 
			url: server localUrl; addPath: 'one'; 
			entity: (ZnEntity text: 'BODY');
			post.
		self assert: client isSuccess.
		self assert: client contents = 'OK!!'.
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testSpecialPosts
	self skip.
	self withServerDo: [ :server |
		{ ZnEntity with: 'A'. ZnEntity with: #[ 42 ]. ZnEntity with: ''. ZnEntity with: #[ ] } 
			do: [ :each |
				| client |
				(client := ZnClient new)
					url: server localUrl;
					addPath: #echo;
					entity: each;
					post.
				self assert: client isSuccess.
				self assert: (each isEmpty or: [client contents includesSubstring: each printString ]).
				client close.
				(client := ZnClient new)
					beOneShot;
					url: server localUrl;
					addPath: #echo;
					entity: each;
					post.
				self assert: client isSuccess.
				self assert: (each isEmpty or: [client contents includesSubstring: each printString ]) ] ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testTimeout
	self withServerDo: [ :server |
		self 
			should: [ 
				ZnClient new 
					timeout: 1; 
					url: server localUrl;
					addPath: 'echo';
					queryAt: 'delay' put: '2'; 
					get ]
			raise: ConnectionTimedOut ]! !

!ZnClientTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/1/2018 23:51'!
testUploadSmallDocument
	self withServerDo: [ :server | | client contents path |
		path := ZnFileSystemUtils fullNameFor: 'small.txt'.
		contents := String streamContents: [ :stream |
			stream print: DateAndTime now; space; print: 9999 atRandom ].
		ZnFileSystemUtils deleteIfExists: 'small.txt'.
		ZnFileSystemUtils writeStreamFor: path do: [ :stream | stream nextPutAll: contents ].
		(client := ZnClient new)
			url: server localUrl;
			addPath: 'echo';
			uploadEntityFrom: path.
		self assert: client request entity contentType = ZnMimeType textPlain.
		client contentType: ZnMimeType textPlain. "Not needed, just test the code path"
		self assert: client request entity contentType = ZnMimeType textPlain.
		client post.
		self assert: client isSuccess.
		self assert: client entity contentType = ZnMimeType textPlain.
		self assert: (client contents includesSubstring: contents).
		client close.
		ZnFileSystemUtils deleteIfExists: 'small.txt' ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testUsernamePassword
	self withServerDo: [ :server | | client |
		server authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
		(client := ZnClient new) 
			url: server localUrl;
			addPathSegment: 'echo';
			get.
		self deny: client isSuccess.
		self assert: client response isAuthenticationRequired.
		client 
			username: 'foo' password: 'secret'; 
			get.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'Zinc').
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testUsernamePasswordInUserInfo
	self withServerDo: [ :server | | url client |
		server authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
		url := server localUrl.
		url username: 'foo'; password: 'secret'; addPathSegment: 'echo'.
		(client := ZnClient new) 
			get: url.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'Zinc').
		client close ]! !

!ZnClientTests methodsFor: 'testing' stamp: ''!
testUsernamePasswordTwice
	self withServerDo: [ :server | | client |
		server authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
		(client := ZnClient new) 
			url: server localUrl;
			addPathSegment: 'echo';
			username: 'foo' password: 'secret'; 
			get.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'Zinc').
		client get.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'Zinc').
		client close ]! !

!ZnClientTests methodsFor: 'private' stamp: ''!
withServerDo: block
	| server |
	server := ZnServer on: 1700 + 32 atRandom.
	[ 
		server start.
		self 
			assert: server isRunning & server isListening
			description: ('Failed to start server on port {1}. Is there one already?' format: { server port }).
		block cull: server
	] 
		ensure: [ server stop ]! !

TestCase subclass: #ZnDigestAuthenticatorTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnDigestAuthenticatorTests methodsFor: 'testing' stamp: ''!
testParseEmpty
	| keys headerValue |
	headerValue := ' Digest username="Mufasa",
                     realm="testrealm@host.com",
                     nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093",
                     uri="/dir/index.html",
                     qop=,
                     nc="",
                     cnonce="0a4f113b",
                     response="6629fae49393a05397450978507c4ef1",
                     opaque="5ccc069c403ebaf9f0171e9517f40e41"'.
	keys := ZnDigestAuthenticator parseAuthRequest: headerValue.
	self assert: (keys at: 'username') equals: 'Mufasa'.
	self assert: (keys at: 'realm') equals: 'testrealm@host.com'.
	self assert: (keys at: 'qop') equals: ''.
	self assert: (keys at: 'nc') equals: ''.
	self assert: (keys at: 'cnonce') equals: '0a4f113b'.
	self assert: (keys at: 'response') equals: '6629fae49393a05397450978507c4ef1'.
	! !

!ZnDigestAuthenticatorTests methodsFor: 'testing' stamp: ''!
testParseOK
	| keys headerValue |
	headerValue := ' Digest username="Mufasa",
                     realm="testrealm@host.com",
                     nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093",
                     uri="/dir/index.html",
                     qop=auth,
                     nc=00000001,
                     cnonce="0a4f113b",
                     response="6629fae49393a05397450978507c4ef1",
                     opaque="5ccc069c403ebaf9f0171e9517f40e41"'.
	keys := ZnDigestAuthenticator parseAuthRequest: headerValue.
	self assert: (keys at: 'username') equals: 'Mufasa'.
	self assert: (keys at: 'realm') equals: 'testrealm@host.com'.
	self assert: (keys at: 'qop') equals: 'auth'.
	self assert: (keys at: 'nc') equals: '00000001'.
	self assert: (keys at: 'cnonce') equals: '0a4f113b'.
	self assert: (keys at: 'response') equals: '6629fae49393a05397450978507c4ef1'.
	! !

TestCase subclass: #ZnDispatcherDelegateTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnDispatcherDelegateTest methodsFor: 'testing' stamp: ''!
testDispatcherDelegate
	self withServerDo: [ :server |  | client |
		server delegate: (ZnDispatcherDelegate new 
			map: '/hello' to: [ :request :response | response entity: (ZnStringEntity html: '<h1>hello server</h1>') ];
			map: '/' to: [ :request :response | response entity: (ZnStringEntity html: '<h1>default</h1>') ]).

		client := ZnClient new.
		
		client get: (server localUrl addPathSegment: 'missing'; yourself).
		self assert: (client response contentType = ZnMimeType textPlain).
		self assert: (client response status = 404).
		
		client get: server localUrl.
		self assert: (client response contentType = ZnMimeType textHtml).
		self assert: (client response status = 200).
		
		client get: (server localUrl addPathSegment: 'hello'; yourself).
		self assert: (client response contentType = ZnMimeType textHtml).
		self assert: (client response status = 200).
		
		client close ]! !

!ZnDispatcherDelegateTest methodsFor: 'private' stamp: ''!
withServerDo: block
	| server |
	server := ZnServer on: 1700 + 32 atRandom.
	[ 
		server start.
		self 
			assert: server isRunning & server isListening
			description: ('Failed to start server on port {1}. Is there one already?' format: { server port }).
		self assert: server isRunning & server isListening.
		block cull: server
	] 
		ensure: [ server stop ]! !

TestCase subclass: #ZnEasyTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnEasyTests methodsFor: 'accessing' stamp: ''!
smallHtmlUrl
	^ 'http://zn.stfx.eu/zn/small.html' asZnUrl! !

!ZnEasyTests methodsFor: 'testing' stamp: ''!
testDelete
	self withServerDo: [ :server | | response |
		response := ZnEasy delete: (server localUrl addPathSegments: #('echo' 'foo'); yourself).
		self assert: (response headers contentType = ZnMimeType textPlain).
		self assert: (response statusLine code = 200).
		self assert: (response entity string includesSubstring: 'DELETE').
		self assert: (response entity string includesSubstring: 'foo') ]! !

!ZnEasyTests methodsFor: 'testing' stamp: ''!
testGetGif
	| url result |
	url := 'http://zn.stfx.eu/zn/Hot-Air-Balloon.gif'.
	result := ZnEasy getGif: url.
	self assert: result isForm! !

!ZnEasyTests methodsFor: 'testing' stamp: ''!
testGetJpeg
	| url result |
	url := 'http://stfx.eu/sun-fire-x2100.jpg'.
	result := ZnEasy getJpeg: url.
	self assert: result isForm! !

!ZnEasyTests methodsFor: 'testing' stamp: ''!
testGetPng
	| url result |
	url := 'http://pharo.org/files/pharo.png'.
	result := ZnEasy getPng: url.
	self assert: result isForm! !

!ZnEasyTests methodsFor: 'testing' stamp: ''!
testGetSmallHTMLDocument
	| url response |
	url := self smallHtmlUrl.
	response := ZnEasy get: url.
	self assert: (response headers contentType = ZnMimeType textHtml).
	self assert: (response statusLine code = 200).
	self assert: (response entity string includesSubstring: 'small')! !

!ZnEasyTests methodsFor: 'testing' stamp: ''!
testHeadSmallHTMLDocument
	| url response |
	url := self smallHtmlUrl.
	response := ZnEasy head: url.
	self assert: (response headers contentType = ZnMimeType textHtml).
	self assert: (response statusLine code = 200).
	self assert: response hasEntity not! !

!ZnEasyTests methodsFor: 'testing' stamp: ''!
testPost
	self withServerDo: [ :server | | response data |
		data := String streamContents: [ :stream | 1 to: 32 do: [ :each | stream nextPut: 'abc' atRandom ] ].
		response := ZnEasy post: (server localUrl addPathSegment: 'echo'; yourself) data: (ZnEntity text: data).
		self assert: (response headers contentType = ZnMimeType textPlain).
		self assert: (response statusLine code = 200).
		self assert: (response entity string includesSubstring: 'POST').
		self assert: (response entity string includesSubstring: data) ]! !

!ZnEasyTests methodsFor: 'testing' stamp: ''!
testPostUnicodeUtf8
	self withServerDo: [ :server | | response data |
		data := String streamContents: [ :stream | 0 to: 16r024F do: [ :each | stream nextPut: each asCharacter ] ].
		response := ZnEasy post: (server localUrl addPathSegment: 'echo'; yourself) data: (ZnEntity text: data).
		self assert: (response headers contentType = ZnMimeType textPlain).
		self assert: (response statusLine code = 200).
		self assert: (response entity string includesSubstring: 'POST').
		self assert: (response entity string includesSubstring: data) ]! !

!ZnEasyTests methodsFor: 'testing' stamp: ''!
testPut
	self withServerDo: [ :server | | response data |
		data := String streamContents: [ :stream | 1 to: 32 do: [ :each | stream nextPut: 'abc' atRandom ] ].
		response := ZnEasy put: (server localUrl addPathSegment: 'echo'; yourself) data: (ZnEntity text: data).
		self assert: (response headers contentType = ZnMimeType textPlain).
		self assert: (response statusLine code = 200).
		self assert: (response entity string includesSubstring: 'PUT').
		self assert: (response entity string includesSubstring: data) ]! !

!ZnEasyTests methodsFor: 'testing' stamp: ''!
testTimeout
	self withServerDo: [ : server |
		self 
			should: [ 
				ZnConnectionTimeout 
					value: 1 
					during: [ ZnEasy get: (server localUrl addPathSegment: 'echo'; queryAt: 'delay' put: '2'; yourself) ] ] 
			raise: ConnectionTimedOut ]
	! !

!ZnEasyTests methodsFor: 'private' stamp: ''!
withServerDo: block
	| server |
	server := ZnServer on: 1700 + 32 atRandom.
	[ 
		server start.
		self 
			assert: server isRunning & server isListening
			description: ('Failed to start server on port {1}. Is there one already?' format: { server port }).
		self assert: server isRunning & server isListening.
		block cull: server
	] 
		ensure: [ server stop ]! !

TestCase subclass: #ZnEntityReaderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnEntityReaderTests methodsFor: 'testing' stamp: ''!
testChunked
	| input headers reader entity |
	input := '3', String crlf,
		'con', String crlf,
		'8', String crlf,
		'sequence', String crlf,
		'0', String crlf, String crlf.
	(headers := ZnHeaders new)
		at: 'Content-Type' put: 'text/plain';
		at: 'Transfer-Encoding' put: 'chunked'.
	(reader := ZnEntityReader new)
		headers: headers;
		stream: input asByteArray readStream.
	entity := reader readEntity.
	self assert: entity contents = 'consequence'! !

!ZnEntityReaderTests methodsFor: 'testing' stamp: ''!
testChunkedWithExtraHeaders
	| input headers reader entity |
	input := '3', String crlf,
		'con', String crlf,
		'8', String crlf,
		'sequence', String crlf,
		'0', String crlf, 
		'X-Foo:bar', String crlf, 
		String crlf.
	(headers := ZnHeaders new)
		at: 'Content-Type' put: 'text/plain';
		at: 'Transfer-Encoding' put: 'chunked'.
	(reader := ZnEntityReader new)
		headers: headers;
		stream: input asByteArray readStream.
	entity := reader readEntity.
	self assert: entity contents = 'consequence'.
	self assert: (headers at: 'X-Foo') = 'bar'! !

!ZnEntityReaderTests methodsFor: 'testing' stamp: ''!
testReadStreaming
	| data headers reader entity |
	data := ((1 to: 32) collect: [ :each | (0 to: 9) atRandom ]) asByteArray.
	(headers := ZnHeaders new)
		at: 'Content-Type' put: ZnMimeType applicationOctetStream printString;
		at: 'Content-Length' put: data size printString.
	(reader := ZnEntityReader new)
		headers: headers;
		stream: data readStream;
		streaming.
	entity := reader readEntity.
	self assert: entity contentType = ZnMimeType applicationOctetStream.
	self assert: entity contentLength = data size.
	self assert: entity contents = data! !

!ZnEntityReaderTests methodsFor: 'testing' stamp: ''!
testReadStreamingWriting
	| data headers reader entity output |
	data := ((1 to: 32) collect: [ :each | (0 to: 9) atRandom ]) asByteArray.
	(headers := ZnHeaders new)
		at: 'Content-Type' put: ZnMimeType applicationOctetStream printString;
		at: 'Content-Length' put: data size printString.
	(reader := ZnEntityReader new)
		headers: headers;
		stream: data readStream;
		streaming.
	entity := reader readEntity.
	self assert: entity contentType = ZnMimeType applicationOctetStream.
	self assert: entity contentLength = data size.
	output := ByteArray streamContents: [ :stream | entity writeOn: stream ].
	self assert: output = data! !

TestCase subclass: #ZnEntityTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testApplicationUrlEncodingAddAll
	| entity string data |
	data := Dictionary new.
	data at: 'foo' put: 1 asString.
	data at: 'bar' put: 2 asString.
	entity := ZnApplicationFormUrlEncodedEntity new.
	"Asking for the content length will force an internal computation of the representation"
	self assert: entity contentLength = 0.
	entity addAll: data.
	string := String streamContents: [ :stream | entity writeOn: stream ].
	self assert: entity contentLength > 0.
	entity := ZnEntity 
		readFrom: string readStream 
		usingType: ZnApplicationFormUrlEncodedEntity designatedMimeType 
		andLength: string size.
	self assert: (entity at: 'foo') equals: '1'.
	self assert: (entity at: 'bar') equals: '2'! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testMultiPartFormDataWriteRead
	| input output bytes |
	input := (ZnMultiPartFormDataEntity new)
		addPart: (ZnMimePart fieldName: 'extra' value: 'my-extra');
		addPart: (ZnMimePart fieldName: 'info' value: 'my-info');
		addPart: (ZnMimePart fieldName: 'file' fileName: 'foo.txt' entity: (ZnEntity text: 'Zinc HTTP Components'));
		yourself.
	self assert: input contentLength notNil.
	self assert: input contentLength > 0.
	self assert: (input contentType matches: ZnMimeType multiPartFormData).
	bytes := ByteArray streamContents: [ :stream | input writeOn: stream ].
	output := (ZnMultiPartFormDataEntity type: input contentType) readFrom: bytes readStream.
	self assert: (output partNamed: 'extra') fieldValue = 'my-extra'.
	self assert: (output partNamed: 'info') fieldValue = 'my-info'.
	self assert: (output partNamed: 'extra') fieldValueString = 'my-extra'.
	self assert: (output partNamed: 'info') fieldValueString = 'my-info'.
	self assert: (output partNamed: 'file') contents = 'Zinc HTTP Components'.
	output := (ZnMultiPartFormDataEntity type: input contentType length: input contentLength) readFrom: bytes readStream.
	self assert: (output partNamed: 'extra') fieldValue = 'my-extra'.
	self assert: (output partNamed: 'info') fieldValue = 'my-info'.
	self assert: (output partNamed: 'extra') fieldValueString = 'my-extra'.
	self assert: (output partNamed: 'info') fieldValueString = 'my-info'.
	self assert: (output partNamed: 'file') contents = 'Zinc HTTP Components'.

	! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testMultiPartFormDataWriteReadBinary
	| input output bytes |
	input := (ZnMultiPartFormDataEntity new)
		addPart: (ZnMimePart fieldName: 'extra' value: 'my-extra');
		addPart: (ZnMimePart fieldName: 'info' value: 'my-info');
		addPart: (ZnMimePart fieldName: 'file' fileName: 'foo.txt' entity: (ZnEntity text: 'Zinc HTTP Components'));
		yourself.
	self assert: input contentLength notNil.
	self assert: input contentLength > 0.
	self assert: (input contentType matches: ZnMimeType multiPartFormData).
	bytes := ByteArray streamContents: [ :stream | input writeOn: stream ].
	output := ZnEntity readBinaryFrom: bytes readStream usingType: input contentType andLength: bytes size.
	self assert: (output partNamed: 'extra') fieldValue = 'my-extra' asByteArray.
	self assert: (output partNamed: 'info') fieldValue = 'my-info' asByteArray.
	self assert: (output partNamed: 'extra') fieldValueString = 'my-extra'.
	self assert: (output partNamed: 'info') fieldValueString = 'my-info'.
	self deny: (output partNamed: 'file') contents isString.
	self assert: (output partNamed: 'file') contents = 'Zinc HTTP Components' asByteArray! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testReading
	| contents entity |
	contents := 'This is a test'.
	entity := ZnEntity 
				readFrom: contents asByteArray readStream 
				usingType: ZnMimeType textPlain 
				andLength: contents size.
	self assert: entity contents isString.
	self assert: entity string = contents.
	self assert: entity contentLength = contents size.
	self assert: entity contentType = ZnMimeType textPlain ! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testReadingApplicationFormUrlEncoding
	| input entity |
	input := #(111 97 117 116 104 95 116 111 107 101 110 95 115 101 99 114 101 116 61 98 121 112 104 57 99 109 104 106 49 53 101 108 121 56 38 111 97 117 116 104 95 116 111 107 101 110 61 56 54 101 112 106 51 116 118 100 49 107 115 120 111 57)
		asByteArray.
	entity := ZnEntity readFrom: input readStream usingType: ZnMimeType applicationFormUrlEncoded andLength: input size.
	self deny: entity isEmpty.
	self assert: (entity includesField: 'oauth_token_secret').
	self assert: (entity includesField: 'oauth_token').
	self assert: (entity at: 'oauth_token_secret') equals: 'byph9cmhj15ely8'.
	self assert: (entity at: 'oauth_token') equals: '86epj3tvd1ksxo9'! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testReadingApplicationFormUrlEncodingNoLength
	| input entity |
	input := #(111 97 117 116 104 95 116 111 107 101 110 95 115 101 99 114 101 116 61 98 121 112 104 57 99 109 104 106 49 53 101 108 121 56 38 111 97 117 116 104 95 116 111 107 101 110 61 56 54 101 112 106 51 116 118 100 49 107 115 120 111 57)
		asByteArray.
	entity := ZnEntity readFrom: input readStream usingType: ZnMimeType applicationFormUrlEncoded andLength: nil.
	self deny: entity isEmpty.
	self assert: (entity includesField: 'oauth_token_secret').
	self assert: (entity includesField: 'oauth_token').
	self assert: (entity at: 'oauth_token_secret') equals: 'byph9cmhj15ely8'.
	self assert: (entity at: 'oauth_token') equals: '86epj3tvd1ksxo9'! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testReadingBinary
	| contents entity |
	contents := 'This is a test'.
	entity := ZnEntity 
				readBinaryFrom: contents asByteArray readStream 
				usingType: ZnMimeType textPlain 
				andLength: contents size.
	self deny: entity contents isString.
	self assert: entity contents = contents asByteArray.
	self assert: entity contentLength = contents size.
	self assert: entity contentType = ZnMimeType textPlain ! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testReadingLimited
	| contents |
	contents := String new: 1025 withAll: $X.
	ZnMaximumEntitySize value: 1024 during: [
		self should: [
			ZnEntity 
				readFrom: contents asByteArray readStream 
				usingType: ZnMimeType textPlain 
				andLength: contents size ]
			raise: ZnEntityTooLarge.
		self should: [
			ZnEntity 
				readFrom: contents asByteArray readStream 
				usingType: ZnMimeType textPlain 
				andLength: nil ]
			raise: ZnEntityTooLarge ] ! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testStringEntityEncoderInitialization
	| entity response |
	entity := ZnEntity text: 'Regression test'.
	self assert: entity hasEncoder not.
	response := ZnResponse ok: entity.
	self assert: entity hasEncoder.
	ByteArray streamContents: [ :stream | response writeOn: stream ].
	self assert: entity hasEncoder
	! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testUTF8ReadingDetermined
	| string entity bytes |
	string := String with: $$ with: (16r00A2 asCharacter) with: ( 16r20AC asCharacter) with: (16r024B62 asCharacter).
	entity := ZnStringEntity type: ZnMimeType textPlain length: 10.
	bytes := #(16r24 16rC2 16rA2 16rE2 16r82 16rAC 16rF0 16rA4 16rAD 16rA2) asByteArray.
	entity readFrom: bytes readStream.
	self assert: entity contentType = ZnMimeType textPlain.
	self assert: entity contentLength = 10.
	self assert: entity contents = string! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testUTF8ReadingUndetermined
	| string entity bytes |
	string := String with: $$ with: (16r00A2 asCharacter) with: ( 16r20AC asCharacter) with: (16r024B62 asCharacter).
	entity := ZnStringEntity type: ZnMimeType textPlain.
	bytes := #(16r24 16rC2 16rA2 16rE2 16r82 16rAC 16rF0 16rA4 16rAD 16rA2) asByteArray.
	entity readFrom: bytes readStream.
	self assert: entity contentType = ZnMimeType textPlain.
	self assert: entity contentLength = 10.
	self assert: entity contents = string! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testUTF8Writing
	| string entity bytes |
	string := String with: $$ with: (16r00A2 asCharacter) with: ( 16r20AC asCharacter) with: (16r024B62 asCharacter).
	entity := ZnStringEntity text: string.
	self assert: entity contentType = ZnMimeType textPlain.
	self assert: entity contentLength = 10.
	bytes := ByteArray streamContents: [ :stream | entity writeOn: stream ].
	self assert: bytes = #(16r24 16rC2 16rA2 16rE2 16r82 16rAC 16rF0 16rA4 16rAD 16rA2) asByteArray! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testUnspecifiedEncoding
	| json bytes entity |
	json := '{"text":"Der Weg zur Hölle ist mit guten Vorsätzen gepflastert."}'.
	entity := ZnEntity with: json type: ZnMimeType applicationJson setCharSetUTF8.
	bytes := entity asByteArray.
	ZnDefaultCharacterEncoder 
		value: ZnUTF8Encoder new
		during: [
			entity := ZnEntity 
				readFrom: bytes readStream 
				usingType: ZnMimeType applicationJson 
				andLength: nil. 
			self assert: entity contents equals: json.
			entity := ZnEntity with: json type: ZnMimeType applicationJson.
			self assert: entity encoder equals: ZnUTF8Encoder new ]! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testWritingApplicationUrlEncoding
	| entity string |
	entity := ZnApplicationFormUrlEncodedEntity new.
	entity at: 'foo' put: 'bar'.
	string := String streamContents: [ :stream | entity writeOn: stream ].
	self assert: string = 'foo=bar'.
	self assert: entity contentLength = 7! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testWritingApplicationUrlEncodingWithTextEncodingLatin1
	| entity string |
	entity := ZnApplicationFormUrlEncodedEntity new.
	entity contentType charSet: 'iso-8859-1'.
	entity at: 'foo' put: (Character value: 246) asString.
	string := String streamContents: [ :stream | entity writeOn: stream ].
	self assert: string = 'foo=%F6'.
	self assert: entity contentLength = 7
	! !

!ZnEntityTests methodsFor: 'testing' stamp: ''!
testWritingApplicationUrlEncodingWithTextEncodingUtf8
	| entity string |
	entity := ZnApplicationFormUrlEncodedEntity new.
	entity contentType charSet: 'utf-8'.
	entity at: 'foo' put: (Character value: 246) asString.
	string := String streamContents: [ :stream | entity writeOn: stream ].
	self assert: string = 'foo=%C3%B6'.
	self assert: entity contentLength = 10! !

TestCase subclass: #ZnEntityWriterTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnEntityWriterTests methodsFor: 'testing' stamp: ''!
testChunkedOnly
	| entity headers writer output reader result |
	entity := 	ZnEntity text: (String streamContents: [ :out |
		{ Collection. Stream. Exception. Magnitude } do: [ :cls |
			cls allSubclassesDo: [ :each | out nextPutAll: each comment ] ] ]).
	headers := ZnHeaders new
		acceptEntityDescription: entity;
		at: 'Transfer-Encoding' put: 'chunked';
		yourself.
	output := ByteArray streamContents: [ :out |
		writer := ZnEntityWriter new
			stream: out;
			headers: headers;
			yourself.
		writer writeEntity: entity ].
	reader := ZnEntityReader new
		stream: output readStream;
		headers: headers;
		yourself.
	result := reader readEntity.
	self assert: result contents equals: entity contents! !

!ZnEntityWriterTests methodsFor: 'testing' stamp: ''!
testGzippedAndChunked
	| entity headers writer output reader result |
	entity := 	ZnEntity text: (String streamContents: [ :out |
		{ Collection. Stream. Exception. Magnitude } do: [ :cls |
			cls allSubclassesDo: [ :each | out nextPutAll: each comment ] ] ]).
	headers := ZnHeaders new
		acceptEntityDescription: entity;
		at: 'Content-Encoding' put: 'gzip';
		at: 'Transfer-Encoding' put: 'chunked';
		yourself.
	output := ByteArray streamContents: [ :out |
		writer := ZnEntityWriter new
			stream: out;
			headers: headers;
			yourself.
		writer writeEntity: entity ].
	reader := ZnEntityReader new
		stream: output readStream;
		headers: headers;
		yourself.
	result := reader readEntity.
	self assert: result contents equals: entity contents! !

!ZnEntityWriterTests methodsFor: 'testing' stamp: ''!
testPlain
	| entity writer output |
	entity := ZnEntity text: 'Foo bar!!'.
	output := ByteArray streamContents: [ :out |
		writer := ZnEntityWriter new
			stream: out;
			headers: ZnHeaders new;
			yourself.
		writer writeEntity: entity ].
	self assert: output equals: 'Foo bar!!' asByteArray! !

!ZnEntityWriterTests methodsFor: 'testing' stamp: ''!
testPlainUsingReader
	| entity headers writer output reader result |
	entity := ZnEntity text: 'Foo bar!!'.
	headers := ZnHeaders new 
		acceptEntityDescription: entity; 
		yourself.
	output := ByteArray streamContents: [ :out |
		writer := ZnEntityWriter new
			stream: out;
			headers: headers;
			yourself.
		writer writeEntity: entity ].
	reader := ZnEntityReader new
		stream: output readStream;
		headers: headers;
		yourself.
	result := reader readEntity.
	self assert: result contents equals: 'Foo bar!!'! !

TestCase subclass: #ZnHeadersTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnHeadersTests methodsFor: 'testing' stamp: ''!
testMergeValued
	| headers |
	headers := ZnHeaders new.
	headers at: 'Set-Cookie' put: 'foo=1'.
	headers at: 'Set-Cookie' put: 'bar=1' ifPresentMerge: [ :old :new | old, ';', new ].
	self assert: (headers at: 'Set-Cookie') equals: 'foo=1;bar=1'
	
	! !

!ZnHeadersTests methodsFor: 'testing' stamp: ''!
testMultiValued
	| headers |
	headers := ZnHeaders new.
	headers at: 'Set-Cookie' put: 'foo=1'.
	headers at: 'Set-Cookie' add: 'bar=1'.
	self assert: (headers at: 'Set-Cookie') equals: #('foo=1' 'bar=1')
	
	! !

!ZnHeadersTests methodsFor: 'testing' stamp: ''!
testMultiValuedStreaming
	| headers keysAndValues |
	headers := ZnHeaders new.
	headers at: 'Set-Cookie' put: 'foo=1'.
	headers at: 'Set-Cookie' add: 'bar=1'.
	keysAndValues := OrderedCollection streamContents: [ : stream |
		headers headersDo: [ :key :value |
			stream nextPut: key; nextPut: value ] ].
	self assert: keysAndValues equals: #('Set-Cookie' 'foo=1' 'Set-Cookie' 'bar=1') asOrderedCollection 
	
	! !

!ZnHeadersTests methodsFor: 'testing' stamp: ''!
testNormalization
	| headers |
	headers := ZnHeaders new.
	headers at: 'content-TYPE' put: 'text/plain'.
	self assert: (headers includesKey: 'content-type').
	self assert: (headers includesKey: 'content-TYPE').
	self assert: (headers at: 'Content-Type') equals: 'text/plain'.
	headers at: 'CONTENT-type' put: 'text/html'.
	self assert: (headers at: 'Content-Type') equals: 'text/html'.

	! !

!ZnHeadersTests methodsFor: 'testing' stamp: ''!
testReading
	| string headers |
	string := 'Content-Type: text/plain', String crlf, 'Content-Length: 128', String crlf.
	headers := ZnHeaders readFrom: string asByteArray readStream.
	self assert: (headers includesKey: 'Content-Type').
	self assert: (headers includesKey: 'Content-Length').
	self assert: (headers at: 'Content-Type') equals: 'text/plain'.
	self assert: (headers at: 'Content-Length') equals: '128'.! !

!ZnHeadersTests methodsFor: 'testing' stamp: ''!
testReadingMultiline
	| string headers |
	string := 'Content-Type: text/plain', String crlf, 
		'Long: foo', String crlf, 
		' bar', String crlf, 
		String tab, 'baz', String crlf, 
		'Content-Length: 128', String crlf.
	headers := ZnHeaders readFrom: string asByteArray readStream.
	self assert: (headers includesKey: 'Content-Type').
	self assert: (headers includesKey: 'Content-Length').
	self assert: (headers at: 'Content-Type') equals: 'text/plain'.
	self assert: (headers at: 'Content-Length') equals: '128'.
	self assert: (headers includesKey: #Long).
	self assert: (headers at: #long) equals: 'foo bar	baz'.! !

!ZnHeadersTests methodsFor: 'testing' stamp: ''!
testReadingMultilineMultiValue
	| string headers |
	string := 'Content-Type: text/plain', String crlf, 
		'Long: foo', String crlf, 
		' bar', String crlf, 
		String tab, 'baz', String crlf, 
		'Content-Length: 128', String crlf,
		'Long: more', String crlf.
	headers := ZnHeaders readFrom: string asByteArray readStream.
	self assert: (headers at: #long) equals: #( 'foo bar	baz' 'more').! !

!ZnHeadersTests methodsFor: 'testing' stamp: ''!
testReadingMultilineNonBinary
	| string headers |
	string := 'Foo: 1', String crlf, 
		' 2', String crlf, 
		String tab, '3', String crlf, 
		'Bar: EOF', String crlf.
	headers := ZnHeaders readFrom: string readStream.
	self assert: (headers at: 'Foo') equals: '1 2', String tab, '3'.
	self assert: (headers at: 'Bar') equals: 'EOF'! !

!ZnHeadersTests methodsFor: 'testing' stamp: ''!
testWriting
	| headers string |
	headers := ZnHeaders new 
		at: 'Content-Type' put: 'text/plain'; 
		at: 'Content-Length' put: '128';
		yourself.
	string := String streamContents: [ :stream | headers writeOn: stream ].
	self assert: (string includesSubstring: 'Content-Type: text/plain', String crlf).
	self assert: (string includesSubstring: 'Content-Length: 128', String crlf).
	self assert: (string occurrencesOf: Character cr) = 2.
	self assert: (string occurrencesOf: Character lf) = 2! !

TestCase subclass: #ZnHtmlOutputStreamTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testAttributeEscaping
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html tag: #div attributes: #(query '"literal"') with: 'Foo' ].
	self assert: result equals: '<div query="&quot;literal&quot;">Foo</div>'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testComment
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html comment: 'a comment' ].
	self assert: result equals: '<!!-- a comment -->'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testEncoding
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html tag: #div with: 'élève en Français' ].
	self assert: result equals: '<div>élève en Français</div>'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testEscaping
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html tag: #div with: 'Foo & Bar < 100' ].
	self assert: result equals: '<div>Foo &amp; Bar &lt; 100</div>'.
	result := ZnHtmlOutputStream streamContents: [ :html |
		html << 'Foo & Bar < 100' ].
	self assert: result equals: 'Foo & Bar < 100'.
	result := ZnHtmlOutputStream streamContents: [ :html |
		html str: 'Foo & Bar < 100' ].
	self assert: result equals: 'Foo &amp; Bar &lt; 100'.! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testFormat
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html format: '{1} & {2}' with: { '<'. '>' } ].
	self assert: result equals: '&lt; &amp; >'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testHtml5Tag
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html html5 ].
	self assert: result equals: '<!!DOCTYPE html>'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testNesting
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html tag: #ol do: [ 
			#(one two three) do: [ :each | html tag: #li with: each ] ] ].
	self assert: result equals: '<ol><li>one</li><li>two</li><li>three</li></ol>'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testSelfClosingTag
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html tag: #br ].
	self assert: result equals: '<br/>'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testSmall
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html 
			tag: #h1 with: #Small;
			tag: #p with: 'This is a small HTML document' ].
	self assert: result equals: '<h1>Small</h1><p>This is a small HTML document</p>'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testSmallPage
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html page: 'Small' do: [ 
			html tag: #p with: 'This is a small HTML document' ] ].
	self assert: result equals: '<!!DOCTYPE html><html><head><title>Small</title></head><body><h1>Small</h1><p>This is a small HTML document</p></body></html>'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testTagWithClass
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html tag: #div class: #menu with: 'Foo' ].
	self assert: result equals: '<div class="menu">Foo</div>'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testTagWithEmptyAttributes
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html tag: #div attributes: #(class menu id one readonly nil) with: 'Foo' ].
	self assert: result equals: '<div class="menu" id="one" readonly>Foo</div>'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testTagWithId
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html tag: #div id: #menu with: 'Foo' ].
	self assert: result equals: '<div id="menu">Foo</div>'! !

!ZnHtmlOutputStreamTests methodsFor: 'tests' stamp: ''!
testTagWithMultipleAttributes
	| result |
	result := ZnHtmlOutputStream streamContents: [ :html |
		html tag: #div attributes: #(class menu id one) with: 'Foo' ].
	self assert: result equals: '<div class="menu" id="one">Foo</div>'! !

TestCase subclass: #ZnLimitedReadStreamTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnLimitedReadStreamTests methodsFor: 'testing' stamp: ''!
testBinary
	| data limitedReadStream |
	data := #(0 1 2 3 4 5 6 7 8 9) asByteArray.
	limitedReadStream := ZnLimitedReadStream on: data readStream limit: 5.
	self assert: limitedReadStream upToEnd = #(0 1 2 3 4) asByteArray! !

!ZnLimitedReadStreamTests methodsFor: 'testing' stamp: ''!
testBinaryAll
	| data limitedReadStream |
	data := #(0 1 2 3 4 5 6 7 8 9) asByteArray.
	limitedReadStream := ZnLimitedReadStream on: data readStream limit: data size.
	self assert: limitedReadStream upToEnd = data! !

!ZnLimitedReadStreamTests methodsFor: 'testing' stamp: ''!
testMatch
	| stream |
	stream := ZnLimitedReadStream on: 'ababcabc' readStream limit: 8.
	self assert: (stream match: 'ab').
	self deny: stream atEnd.
	stream := ZnLimitedReadStream on: 'ababcabc' readStream limit: 8.
	self assert: (stream match: 'abc').
	self deny: stream atEnd.
	self assert: (stream match: 'abc').
	self assert: stream atEnd.
	stream := ZnLimitedReadStream on: 'ababcabc' readStream limit: 8.
	self deny: (stream match: 'xx').
	self assert: stream atEnd.
! !

!ZnLimitedReadStreamTests methodsFor: 'testing' stamp: ''!
testNext
	| limitedReadStream |
	limitedReadStream := ZnLimitedReadStream on: String new readStream limit: 10.
	self assert: limitedReadStream atEnd.
	self assert: limitedReadStream next isNil.
	limitedReadStream := ZnLimitedReadStream on: 'ABC' readStream limit: 10.
	self assert: limitedReadStream atEnd not.
	self assert: limitedReadStream next = $A.
	self assert: limitedReadStream peek = $B.
	limitedReadStream next.
	self assert: limitedReadStream next = $C.
	self assert: limitedReadStream next isNil.
	self assert: limitedReadStream atEnd
	! !

!ZnLimitedReadStreamTests methodsFor: 'testing' stamp: ''!
testNextCount
	| data stream |
	data := '0123456789'.
	stream := ZnLimitedReadStream on: data readStream limit: 8.
	self assert: (stream next: 4) = '0123'! !

!ZnLimitedReadStreamTests methodsFor: 'testing' stamp: ''!
testNextCountInto
	| data stream buffer |
	data := '0123456789'.
	stream := ZnLimitedReadStream on: data readStream limit: 8.
	buffer := String new: 4.
	self assert: (stream next: 4 into: buffer) = '0123'.
	self assert: buffer = '0123'! !

!ZnLimitedReadStreamTests methodsFor: 'testing' stamp: ''!
testNextCountIntoShort
	| data stream buffer |
	data := '0123456789'.
	stream := ZnLimitedReadStream on: data readStream limit: 5.
	buffer := String new: 6.
	self assert: (stream next: 6 into: buffer) = '01234'
	! !

!ZnLimitedReadStreamTests methodsFor: 'testing' stamp: ''!
testNextCountShort
	| data stream |
	data := '0123456789'.
	stream := ZnLimitedReadStream on: data readStream limit: 5.
	self assert: (stream next: 6) = '01234'! !

!ZnLimitedReadStreamTests methodsFor: 'testing' stamp: ''!
testSimple
	| data limitedReadStream |
	data := '0123456789'.
	limitedReadStream := ZnLimitedReadStream on: data readStream limit: 5.
	self assert: limitedReadStream upToEnd = '01234'! !

!ZnLimitedReadStreamTests methodsFor: 'testing' stamp: ''!
testSimpleEof
	| data limitedReadStream |
	data := '0123456789'.
	limitedReadStream := ZnLimitedReadStream on: data readStream limit: 15.
	self assert: limitedReadStream upToEnd = '0123456789'.
	self assert: limitedReadStream atEnd.
	self assert: limitedReadStream next isNil! !

TestCase subclass: #ZnLineReaderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnLineReaderTests methodsFor: 'testing' stamp: ''!
testBinary
	| input reader |
	input := ('Foo', String crlf, 'Bar', String crlf) asByteArray.
	reader := ZnLineReader on: input readStream.
	self assert: reader nextLine = 'Foo'.
	self assert: reader nextLine = 'Bar'.
	self assert: reader nextLine isEmpty! !

!ZnLineReaderTests methodsFor: 'testing' stamp: ''!
testLineTooLong
	| input reader |
	input := (String new: 128) atAllPut: $X.
	reader := (ZnLineReader on: input readStream) limit: 64.
	self should: [ reader nextLine ] raise: ZnLineTooLong  ! !

!ZnLineReaderTests methodsFor: 'testing' stamp: ''!
testLineTooLongDefault
	| input reader |
	input := (String new: ZnConstants maximumLineLength + 1) atAllPut: $X.
	reader := ZnLineReader on: input readStream.
	self should: [ reader nextLine ] raise: ZnLineTooLong  ! !

!ZnLineReaderTests methodsFor: 'testing' stamp: ''!
testSimple
	| input reader |
	input := 'Foo', String crlf, 'Bar', String crlf.
	reader := ZnLineReader on: input readStream.
	self assert: reader nextLine = 'Foo'.
	self assert: reader nextLine = 'Bar'.
	self assert: reader nextLine isEmpty! !

TestCase subclass: #ZnMagicCookieJarTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnMagicCookieJarTests methodsFor: 'accessing' stamp: ''!
cookieString
	^ 'PREF=ID=18ae67ce302b6317:TM=1285696140:LM=1285696140:S=hqroFGlRQR1j4Yhy; expires={1}; path=/; domain=.google.com' format: { ZnUtils httpDate: DateAndTime now + 60 days }! !

!ZnMagicCookieJarTests methodsFor: 'accessing' stamp: ''!
cookieStringAlt
	^'PREFS=ID=18ae67ce302b6317:TM=1285696140:LM=1285696140:S=hqroFGlRQR1j4Yhy; expires={1}; path=/; domain=.pharo-project.org' format: { ZnUtils httpDate: DateAndTime now + 60 days }! !

!ZnMagicCookieJarTests methodsFor: 'testing' stamp: ''!
testAdd
	| jar count |
	jar := ZnCookieJar new.
	count := jar cookies size.
	jar add: (ZnCookie fromString: self cookieString for: 'www.google.com' asZnUrl).
	self assert: ((jar cookies size) = (count + 1))! !

!ZnMagicCookieJarTests methodsFor: 'testing' stamp: ''!
testCookieAtForUrl
	| jar c1 c2 |
	jar := ZnCookieJar new.
	c1 := ZnCookie fromString: self cookieString for: 'http://www.google.com' asZnUrl.
	c2 := ZnCookie fromString: self cookieStringAlt for: 'http://www.pharo-project.org' asZnUrl.
	jar add: c1; add: c2.
	self assert: ((jar cookieAt: 'PREF' forUrl: 'http://www.google.com' asZnUrl) notNil).
	self assert: ((jar cookieAt: 'foobar' forUrl: 'http://www.google.com' asZnUrl) isNil)! !

!ZnMagicCookieJarTests methodsFor: 'testing' stamp: ''!
testCookiesForUrl
	| jar c1 c2 |
	jar := ZnCookieJar new.
	c1 := ZnCookie fromString: self cookieString for: 'http://www.google.com' asZnUrl.
	c2 := ZnCookie fromString: self cookieStringAlt for: 'http://www.pharo-project.org' asZnUrl.
	jar add: c1; add: c2.
	self assert: ((jar cookiesForUrl: 'http://www.google.com' asZnUrl) size = 1)! !

TestCase subclass: #ZnMagicCookieTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnMagicCookieTests methodsFor: 'testing' stamp: ''!
cookieString
	^ 'PREF=ID=18ae67ce302b6317:TM=1285696140:LM=1285696140:S=hqroFGlRQR1j4Yhy; expires={1}; path=/; domain=.google.com' format: { ZnUtils httpDate: DateAndTime now + 60 days }! !

!ZnMagicCookieTests methodsFor: 'accessing' stamp: ''!
cookieStringSubpath
	^ 'PREF=ID=18ae67ce302b6317:TM=1285696140:LM=1285696140:S=hqroFGlRQR1j4Yhy; expires={1}; path=/test/; domain=.google.com' format: { ZnUtils httpDate: DateAndTime now + 60 days }! !

!ZnMagicCookieTests methodsFor: 'testing' stamp: ''!
testFromString
	| cookie |
	cookie := ZnCookie fromString: self cookieString for: 'www.google.com' asZnUrl.
	self assert: (cookie name = 'PREF').
	self assert: (cookie path notNil).
	self assert: (cookie domain = '.google.com').
	self assert: (cookie isExpired not)
	! !

!ZnMagicCookieTests methodsFor: 'testing' stamp: ''!
testInDomain
	| cookie |
	cookie := ZnCookie fromString: self cookieString for: 'http://www.google.com' asZnUrl.
	self assert: (cookie isInDomain: 'http://www.google.com' asZnUrl).
	self assert: (cookie isInDomain: 'http://google.com' asZnUrl).
	self assert: ((cookie isInDomain: 'http://pharo-project.org' asZnUrl) not)
	! !

!ZnMagicCookieTests methodsFor: 'testing' stamp: ''!
testInPath
	| cookie |
	cookie := ZnCookie fromString: self cookieStringSubpath for: 'www.google.com' asZnUrl.
	self assert: (cookie isInPath: 'www.google.com/test/' asZnUrl).
	self assert: ((cookie isInPath: 'www.google.com' asZnUrl) not)
	! !

TestCase subclass: #ZnMessageBenchmarkTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnMessageBenchmarkTests methodsFor: 'testing' stamp: ''!
testRequests
	ZnMessageBenchmark requests do: [ :each |
		| bench |
		bench := ZnMessageBenchmark new.
		bench perform: each.
		self assert: (bench message isKindOf: ZnRequest).
		bench writeRepresentation.
		self assert: (bench read isKindOf: ZnRequest).
		self assert: bench read equals: bench message ]
		
	! !

!ZnMessageBenchmarkTests methodsFor: 'testing' stamp: ''!
testResponses
	ZnMessageBenchmark responses do: [ :each |
		| bench |
		bench := ZnMessageBenchmark new.
		bench perform: each.
		self assert: (bench message isKindOf: ZnResponse).
		bench writeRepresentation.
		self assert: (bench read isKindOf: ZnResponse).
		self assert: bench read equals: bench message ]
		
	! !

TestCase subclass: #ZnRequestLineTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnRequestLineTests methodsFor: 'testing' stamp: ''!
testReading
	| requestLine string |
	string := 'GET /foo/bar/xyz.txt HTTP/1.1', String crlf.
	requestLine := ZnRequestLine readFrom: string readStream.
	self assert: requestLine method = #GET.
	self assert: requestLine uriPathQueryFragment = '/foo/bar/xyz.txt'.
	self assert: requestLine version = ZnConstants defaultHTTPVersion ! !

!ZnRequestLineTests methodsFor: 'testing' stamp: ''!
testReadingWrongMethod
	| string |
	string := 'GETT /foo/bar/xyz.txt HTTP/1.1', String crlf.
	self should: [ ZnRequestLine readFrom: string readStream ] raise: ZnUnknownHttpMethod! !

!ZnRequestLineTests methodsFor: 'testing' stamp: ''!
testReadingWrongVersion
	| string |
	string := 'GET /foo/bar/xyz.txt HTTP/1.5', String crlf.
	self should: [ ZnRequestLine readFrom: string readStream ] raise: ZnUnknownHttpVersion! !

!ZnRequestLineTests methodsFor: 'testing' stamp: ''!
testWriting
	| requestLine string |
	requestLine := ZnRequestLine method: 'GET' uri: '/foo/bar/xyz.txt'.
	string := String streamContents: [ :stream | requestLine writeOn: stream ].
	self assert: string = ('GET /foo/bar/xyz.txt HTTP/1.1', String crlf)! !

TestCase subclass: #ZnRequestTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnRequestTests methodsFor: 'testing' stamp: ''!
testBasicAuthenticate
	| request |
	request := ZnRequest new.
	request setBasicAuthenticationUsername: 'user' password: 'secret'.
	self assert: (request headers at: 'Authorization') = 'Basic dXNlcjpzZWNyZXQ='.
	self assert: request basicAuthentication asArray equals: #( 'user' 'secret' ).
	request setBasicAuthenticationUsername: 'user' password: ''.
	self assert: (request headers at: 'Authorization') = 'Basic dXNlcjo='.
	self assert: request basicAuthentication asArray equals: #( 'user' '' ).
	request setBasicAuthenticationUsername: 'user' password: ':colons:'.
	self assert: (request headers at: 'Authorization') = 'Basic dXNlcjo6Y29sb25zOg=='.
	self assert: request basicAuthentication asArray equals: #( 'user' ':colons:' ).! !

!ZnRequestTests methodsFor: 'testing' stamp: ''!
testCookiesParsing
	| input request cookies testCookie1 testCookie2 |
	input := 'GET /foo.html HTTP/1.1', String crlf, 
		'Cookie: testCookie1=123; testCookie2=321', String crlf,
		'Host: foo.com', String crlf, 
		'Agent: SUnit', String crlf, 
		String crlf.
	request := ZnRequest readFrom: input readStream.
	
	cookies := request cookies.
	testCookie1 := cookies detect: [ :each | each name = 'testCookie1' ].
	testCookie2 := cookies detect: [ :each | each name = 'testCookie2' ].
	
	self assert: testCookie1 notNil.
	self assert: testCookie1 value = '123'.	
	
	self assert: testCookie2 notNil.
	self assert: testCookie2 value = '321'.	

! !

!ZnRequestTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2018 21:32'!
testCookiesWriting
	| client testCookie1 testCookie2 cookies |
	client := ZnClient new.
	client url: 'http://www.example.com'.
	(testCookie1 := ZnCookie name: 'testCookie1' value: '123')
		domain: 'example.com'.
	(testCookie2 := ZnCookie name: 'testCookie2' value: '321')
		domain: 'example.com'.
	(client session cookieJar)
		add: testCookie1;
		add: testCookie2.
	client sendCookies.
	cookies := client request cookies do: [ :each | each defaultDomain: 'example.com' ].
	self assert: (cookies includes: testCookie1).
	self assert: (cookies includes: testCookie2)
	
		
! !

!ZnRequestTests methodsFor: 'testing' stamp: ''!
testMergedFields
	| request form mergedFields |
	request := ZnRequest post: 'http://host.com/foo?x=1&y=2&x=3'.
	form := ZnApplicationFormUrlEncodedEntity new.
	form
		at: #z put: '100';
		at: #z add: '200';
		at: #y put: '0'.
	request entity: form.
	mergedFields := request mergedFields.
	self assert: mergedFields keys sorted equals: #(x y z).
	self assert: (mergedFields at: #x) sorted equals: #('1' '3').
	self assert: (mergedFields at: #y) sorted equals: #('0' '2').
	self assert: (mergedFields at: #z) sorted equals: #('100' '200')! !

!ZnRequestTests methodsFor: 'testing' stamp: ''!
testReading
	| input request |
	input := 'GET /foo.html HTTP/1.1', String crlf, 
		'Host: foo.com', String crlf, 
		'Agent: SUnit', String crlf, 
		String crlf.
	request := ZnRequest readFrom: input readStream.
	self assert: request method = #GET.
	self assert: request hasHeaders.
	self assert: (request headers at: 'Host') = 'foo.com'.
	self assert: request hasEntity not
! !

!ZnRequestTests methodsFor: 'testing' stamp: ''!
testReadingWithEntity
	| input request |
	input := 'GET /foo.html HTTP/1.1', String crlf, 
		'Host: foo.com', String crlf, 
		'Content-Length: 8', String crlf, 
		'Content-Type: text/plain', String crlf, 
		String crlf, 
		'12345678'.
	request := ZnRequest readFrom: input asByteArray readStream.
	self assert: request method equals: #GET.
	self assert: request hasHeaders.
	self assert: (request contentType matches: ZnMimeType textPlain).
	self assert: request contentLength equals: 8.
	self assert: request hasHeaders.
	self assert: request entity contents equals: '12345678'
! !

!ZnRequestTests methodsFor: 'testing' stamp: ''!
testUrl
	| url request input |
	url := 'http://zn.stfx.eu/zn/small.thml' asZnUrl.
	request := ZnRequest get: url.
	"The original URL is kept inside the request line object inside the request object"
	self assert: request url equals: url.
	self assert: request relativeUrl equals: url asRelativeUrl.
	self assert: request absoluteUrl equals: url.
	url := 'http://foo.com/foo.html' asZnUrl.
	input := 'GET /foo.html HTTP/1.1' , String crlf , 
		'Host: foo.com' , String crlf , 
		String crlf.
	request := ZnRequest readFrom: input readStream.
	self assert: request url equals: url asRelativeUrl.
	self assert: request relativeUrl equals: url asRelativeUrl.
	self assert: request absoluteUrl equals: url! !

TestCase subclass: #ZnResponseTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testAbsoluteCreated
	| response uri |
	uri := 'http://www.example.com:8888/something/else/foo.txt'.
	response := ZnResponse created: uri asZnUrl.
	self assert: response code = 201.
	self assert: response hasHeaders.
	self assert: (response headers at: 'Location') = uri.
	self assert: response hasEntity.
	self assert: (response entity contents includesSubstring: uri)! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testAbsoluteRedirect
	| response uri |
	uri := 'http://foo.com:8080/something/else/foo.txt?key=123'.
	response := ZnResponse redirect: uri asZnUrl.
	self assert: response code = 302.
	self assert: response isRedirect.
	self assert: response hasHeaders.
	self assert: (response headers at: 'Location') = uri.
	self assert: response hasEntity.
	self assert: (response entity contents includesSubstring: uri)
! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testCookie
	| response cookie1  responseString |
	response := ZnResponse ok: (ZnStringEntity html: '<h1>hello world</h1>').
	cookie1 := ZnCookie new.
	cookie1 
		name: 'testCookie1';
		value: '123'.
		
	response addCookie: cookie1.
	
	responseString := String streamContents: [ :stream | response writeOn: stream ].
	
	self assert: (responseString includesSubstring: 'Set-Cookie: testCookie1=123')	

	
		
		
! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testCookies
	| response cookie1 cookie2 responseString |
	response := ZnResponse ok: (ZnStringEntity html: '<h1>hello world</h1>').
	cookie1 := ZnCookie new.
	cookie1 
		name: 'testCookie1';
		value: '123'.
		
	cookie2 := ZnCookie new.
	cookie2 
		name: 'testCookie2';
		value: '321'.
		
	response addCookie: cookie1.
	response addCookie: cookie2.
	
	responseString := String streamContents: [ :stream | response writeOn: stream ].
	
	self assert: (responseString includesSubstring: 'Set-Cookie: testCookie1=123').  
	self assert: (responseString includesSubstring: 'Set-Cookie: testCookie2=321').  	

	
		
		
! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testCreated
	| response uri |
	uri := '/something/else/foo.txt'.
	response := ZnResponse created: uri.
	self assert: response code = 201.
	self assert: response hasHeaders.
	self assert: (response headers at: 'Location') = uri.
	self assert: response hasEntity.
	self assert: (response entity contents includesSubstring: uri)! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testDefaultUTF8Encoding
	| json entity response bytes result |
	"Even if we do not specificy UTF-8 explicitly, it should be the default"
	json := '{''bank'':''Région Genève Rhône''}'.
	entity := ZnEntity with: json type: 'application/json'.
	response := ZnResponse ok: entity.
	bytes := ByteArray streamContents: [ :out | response writeOn: out ].
	result := ZnResponse readFrom: bytes readStream.
	self assert: result entity contents equals: json! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testIsError
	| response |
	response := ZnResponse ok: (ZnStringEntity html: '<h1>hello world</h1>').
	self assert: response isError not.
	response := ZnResponse notFound: 'http:///secret.txt'.
	self assert: response isError.
	response := ZnResponse redirect: '/something/else/foo.txt'.
	self assert: response isError not! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testNotFound
	| response |
	response := ZnResponse notFound: 'http:///secret.txt'.
	self assert: response code = 404.
	self assert: (response headers includesKey: 'Server').
	self assert: (response headers includesKey: 'Date').
	self assert: (response entity contents includesSubstring: 'secret.txt')! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testRedirect
	| response uri |
	uri := '/something/else/foo.txt'.
	response := ZnResponse redirect: uri.
	self assert: response code = 302.
	self assert: response code = 302.
	self assert: response hasHeaders.
	self assert: (response headers at: 'Location') = uri.
	self assert: response hasEntity.
	self assert: (response entity contents includesSubstring: uri).
	response := ZnResponse redirect: uri asZnUrl.
	self assert: response code = 302.
	self assert: response code = 302.
	self assert: response hasHeaders.
	self assert: (response headers at: 'Location') = uri.
	self assert: response hasEntity.
	self assert: (response entity contents includesSubstring: uri)! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testSlashdotGzipChunked
	| request stream response contents |
	(request := ZnRequest get: 'http://slashdot.org')
		setConnectionClose;
		setAcceptEncodingGzip.
	stream := ZnNetworkingUtils socketStreamToUrl: request url.
	response := [ 
  		request writeOn: stream. 
  		stream flush. 
  		ZnDefaultCharacterEncoder 
			value: ZnNullEncoder new
			during: [ ZnResponse readFrom: stream ] ] ensure: [ stream close ].
	response isRedirect ifTrue: [ ^ self ].
	self assert: response code = 200.
	self assert: ((response headers at: 'Content-Type') beginsWith: 'text/html').
	self assert: (response headers at: 'Content-Encoding' ifAbsent: [ 'gzip' ]) = 'gzip'.
	self assert: (response headers at: 'Transfer-Encoding' ifAbsent: [ 'chunked' ]) = 'chunked'.
	contents := response entity contents.
	self assert: ((contents includesSubstring: 'Slashdot') or: [ contents includesSubstring: 'refresh' ])
! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testWritingBinary
	| response bytes string |
	response := ZnResponse ok: (ZnEntity text: 'Okay').
	bytes := ByteArray streamContents: [ :stream | response writeOn: stream ].
	string := bytes asString.
	self assert: (string includesSubstring: '200 OK').
	self assert: (string includesSubstring: 'Okay')! !

!ZnResponseTests methodsFor: 'testing' stamp: ''!
testWritingCharacter
	| response string |
	response := ZnResponse ok: (ZnEntity text: 'Okay').
	string := String streamContents: [ :stream | response writeOn: stream ].
	self assert: (string includesSubstring: '200 OK').
	self assert: (string includesSubstring: 'Okay')! !

TestCase subclass: #ZnServerTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnServerTests methodsFor: 'testing' stamp: ''!
assertEntityTooLarge: serverUrl
	| response client |
	client := ZnClient new
		url: serverUrl;
		addPathSegment: #echo;
		entity: (ZnEntity with: (ByteArray new: self entitySizeLimit + 1));
		yourself.
	response := client
		post;
		response.
	client close.
	self deny: response isSuccess.
	self assert: response code equals: 400.
	self assert: response contentType equals: ZnMimeType textPlain.
	self assert: (response entity contents includesSubstring: 'ZnEntityTooLarge')! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
assertHeaderLineTooLong: serverUrl
	| response client |
	client := ZnClient new
		url: serverUrl;
		addPathSegment: #echo;
		headerAt: 'X-Test' put: (String new: ZnConstants maximumLineLength withAll: $X);
		yourself.
	response := client
		get;
		response.
	client close.
	self deny: response isSuccess.
	self assert: response code equals: 400.
	self assert: response contentType equals: ZnMimeType textPlain.
	self assert: (response entity contents includesSubstring: 'ZnLineTooLong')! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
assertRequestLineTooLong: serverUrl
	| url response |
	url := serverUrl
		addPathSegment: #echo;
		addPathSegment: (String new: ZnConstants maximumLineLength withAll: $X);
		yourself.
	response := ZnEasy get: url.
	self deny: response isSuccess.
	self assert: response code equals: 400.
	self assert: response contentType equals: ZnMimeType textPlain.
	self assert: (response entity contents includesSubstring: 'ZnLineTooLong')! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
assertTooManyHeaders: serverUrl
	| response client |
	client := ZnClient new
		url: serverUrl;
		addPathSegment: #echo;
		yourself.
	client request headers unlimited.
	1 to: 256 do: [ :each | client headerAt: 'X-Test-' , each printString put: each printString ].
	response := client
		get;
		response.
	client close.
	self deny: response isSuccess.
	self assert: response code equals: 400.
	self assert: response contentType equals: ZnMimeType textPlain.
	self assert: (response entity contents includesSubstring: 'ZnTooManyDictionaryEntries')! !

!ZnServerTests methodsFor: 'accessing' stamp: ''!
entitySizeLimit
	^ 1024! !

!ZnServerTests methodsFor: 'private' stamp: ''!
port
	^ 1700 + 32 atRandom! !

!ZnServerTests methodsFor: 'private' stamp: ''!
runningOnWindows
	"Pharo 3+"
	[ ^ OSPlatform current isWindows ] on: MessageNotUnderstood do: [ ].
	"Pharo 2-"
	[ ^ OSPlatform isWin32 ] on: MessageNotUnderstood do: [ ].
	"Give up"
	^ false! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testAuthorizationFailed
	self withServerDo: [ :server | | response |
		server authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
		response := ZnEasy get: (server localUrl addPathSegment: 'echo'; yourself).
		self assert: (response headers contentType = ZnMimeType textPlain).
		self assert: (response statusLine code = 401).
		self assert: ((response headers at: 'WWW-Authenticate') includesSubstring: 'Basic').
		self assert: ((response headers at: 'WWW-Authenticate') includesSubstring: 'Zinc') ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testAuthorizationSuccessful
	self withServerDo: [ :server | | response |
		server authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
		response := ZnEasy get: (server localUrl addPathSegments: #('echo' 'foo'); yourself) username: 'foo' password: 'secret'.
		self assert: (response headers contentType = ZnMimeType textPlain).
		self assert: (response statusLine code = 200).
		self assert: (response entity string includesSubstring: 'Zinc').
		self assert: (response entity string includesSubstring: 'foo') ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testCustomDefaultDelegate
	self withServerDo: [ :server | | customDelegate client |
		customDelegate := ZnDefaultServerDelegate empty.
		customDelegate 
			map: 'ok' to: [ :request | ZnResponse ok: (ZnEntity text: 'OK') ];
			map: '/' to: 'ok';
			map: 'OK' to: [ :request | ZnResponse redirect: '/ok' ].
		server delegate: customDelegate.
		client := ZnClient new url: server localUrl.
		self assert: client get equals: 'OK'.
		client addPath: 'OK'.
		self assert: client get equals: 'OK'.
		client close	 ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testDefault
	| port server initialDefaultServer wasRunning |
	wasRunning := ZnServer default 
		ifNil: [ false ] 
		ifNotNil: [ ZnServer default isRunning ].
	initialDefaultServer := ZnServer stopDefault.
	self assert: ZnServer default isNil.
	port := self port.
	server := ZnServer startDefaultOn: port.
	self assert: ZnServer default notNil.
	self assert: ZnServer default == server.
	self assert: ZnServer default port = port.
	self assert: ZnServer default isRunning.
	self assert: (ZnServer managedServers includes: server).
	ZnServer stopDefault.
	self assert: ZnServer default isNil.
	self deny: server isRunning.
	self deny: (ZnServer managedServers includes: server).
	server := ZnServer startDefaultOn: port.
	"Starting the default again is actually a restart"
	ZnServer startDefaultOn: port.
	self assert: ZnServer default == server.
	ZnServer stopDefault.
	ZnServer adoptAsDefault: initialDefaultServer.
	self assert: initialDefaultServer == ZnServer default.
	wasRunning ifTrue: [ ZnServer default start ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testEcho
	self withServerDo: [ :server | | response |
		response := ZnEasy get: (server localUrl addPathSegments: #('echo' 'foo'); yourself).
		self assert: (response contentType = ZnMimeType textPlain).
		self assert: (response statusLine code = 200).
		self assert: (response entity contents includesSubstring: 'Zinc').
		self assert: (response entity contents includesSubstring: 'foo').
		self assert: (response entity contents includesSubstring: server printString) ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testEchoBinary
	self withServerDo: [ :server | | response entityIn entityOut |
		server reader: [ :stream | ZnRequest readBinaryFrom: stream ].
		entityIn := ZnEntity with: 'ABC' type: 'text/plain'.
		response := ZnEasy put: (server localUrl addPathSegment: 'echo'; yourself)  data: entityIn.
		self assert: (response contentType = ZnMimeType textPlain).
		self assert: (response statusLine code = 200).
		entityOut := ZnEntity with: entityIn string asByteArray type: entityIn contentType.
		self assert: (response entity contents includesSubstring: entityOut printString) ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testEchoLocalInterface
	| server response |
	(server := ZnServer on: self port)
		bindingAddress: NetNameResolver loopBackAddress.
	[ 
		server start.
		self 
			assert: server isRunning & server isListening
			description: ('Failed to start server on port {1}. Is there one already?' format: { server port }).
		response := ZnEasy get: (server localUrl addPathSegments: #('echo' 'foo'); yourself).
		self assert: (response contentType = ZnMimeType textPlain).
		self assert: (response statusLine code = 200).
		self assert: (response entity contents includesSubstring: 'Zinc').
		self assert: (response entity contents includesSubstring: 'foo')
	]
		ensure: [ server stop ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testEntityTooLarge
	self usingClassicSocketStreamsOnWindowsDo: [ 
		self withServerDo: [ :server | 
			server maximumEntitySize: self entitySizeLimit.
			self deny: server debugMode.
			self assertEntityTooLarge: server localUrl ] ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testError
	self withServerDo: [ :server | | url response |
		self deny: server debugMode.
		url := server localUrl
					addPathSegment: #error;
					queryAt: #message put: 'Test Error';
					yourself.
		response := ZnEasy get: url.
		self deny: response isSuccess.
		self assert: response code equals: 500.
		self assert: response contentType equals: ZnMimeType textPlain.
		self assert: (response entity contents includesSubstring: 'Test Error') ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testFormTest1
	| inputs client |
	inputs := #( '1+2=3 & 2+1=3' 'single ''' 'double " - cool ?' 'élèves français' 'Ελλάδα' ).
	self withServerDo: [ :server |
		(client := ZnClient new)
			url: server localUrl;
			addPathSegment: 'form-test-1'.
		inputs do: [ :each  |
			client
				queryAt: #input put: each;
				get.
			self assert: client isSuccess.
			self assert: (client contents includesSubstring: each) ] ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testFormTest2
	| inputs client |
	inputs := #( '1+2=3 & 2+1=3' 'single ''' 'double " - cool ?' 'élèves français' 'Ελλάδα' ).
	self withServerDo: [ :server |
		(client := ZnClient new)
			url: server localUrl;
			addPathSegment: 'form-test-2'.
		inputs do: [ :each  |
			client
				formAt: #input put: each;
				post.
			self assert: client isSuccess.
			self assert: (client contents includesSubstring: each) ] ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testFormTest3
	| inputs client part |
	inputs := #( ('foo.txt' 'this is foo!!') ('français.txt' 'Je te dis: élèves français') ('Ελλάδα.txt' 'Ελλάδα is it!!') ).
	self withServerDo: [ :server |
		server debugMode: true.
		(client := ZnClient new)
			url: server localUrl;
			addPathSegment: 'form-test-3'.
		inputs do: [ :each  |
			part := ZnMimePart 
				fieldName: 'file' 
				fileName: each first 
				entity: (ZnEntity with: each second).
			client
				resetEntity;
				addPart: part;
				post.
			self assert: client isSuccess.
			self assert: (client contents includesSubstring: each first).
			self assert: (client contents includesSubstring: each second) ] ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testFormTest3Unspecified
	"Simulate what most browsers seem to do: upload text files utf-8 encoded 
	without specifying the charset in the mime type of the mime part"
	
	| inputs client part entity |
	inputs := #( ('foo.txt' 'this is foo!!') ('français.txt' 'Je te dis: élèves français') ('Ελλάδα.txt' 'Ελλάδα is it!!') ).
	self withServerDo: [ :server |
		server debugMode: true.
		(client := ZnClient new)
			url: server localUrl;
			addPathSegment: 'form-test-3'.
		inputs do: [ :each  |
			entity := ZnEntity 
				with: each second utf8Encoded 
				type: ZnMimeType textPlain clearCharSet.
			part := ZnMimePart fieldName: 'file' fileName: each first entity: entity.
			client
				resetEntity;
				addPart: part;
				post.
			self assert: client isSuccess.
			self assert: (client contents includesSubstring: each first).
			self assert: (client contents includesSubstring: each second) ] ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testGetConnectionClose
	self withServerDo: [ :server | | client |
		client := ZnClient new.
		client url: server localUrl; addPathSegment: #echo.
		client request setConnectionClose.
		client get.
		self assert: client isSuccess.
		self assert: (client contents includesSubstring: 'echo').
		self assert: client response isConnectionClose.
		client close ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testGetUnicodeUtf8
	self withServerDo: [ :server | | response html |
		response := ZnEasy get: (server localUrl addPathSegment: 'unicode'; yourself).
		self assert: (response contentType = ZnMimeType textHtml).
		self assert: (response statusLine code = 200).
		self assert: response contentType isCharSetUTF8.
		html := response entity contents.
		self assert: (html includesSubstring: 'Unicode').
		0 to: 16r024F do: [ :each | self assert: (html includes: each asCharacter) ] ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testGzipCompressionAndChunking
	self withServerDo: [ :server | | client |
		server useGzipCompressionAndChunking: true.
		(client := ZnClient new)
			url: server localUrl;
			addPathSegment: 'dw-bench';
			setAcceptEncodingGzip;
			get.
		self assert: client isSuccess.
		self assert: client response hasContentEncoding.
		self assert: (client response headers at: 'Content-Encoding') equals: 'gzip'.
		self assert: client response hasTransferEncoding.
		self assert: (client response headers at: 'Transfer-Encoding') equals: 'chunked' ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testHeaderLineTooLong
	self usingClassicSocketStreamsOnWindowsDo: [
		self withServerDo: [ :server |
			self deny: server debugMode.
			self assertHeaderLineTooLong: server localUrl ] ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testLocalUrl
	self withServerDo: [ :server | 
		self assert: server localUrl scheme equals: #http.
		self assert: server localUrl host equals: NetNameResolver loopBackName.
		self assert: server localUrl port equals: server port ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testOSAssignedPort
	| server | 
	server := ZnServer on: 0.
	self assert: server port equals: 0.
	[ 
		| response |
		server start.
		self assert: server port > 0. 
		response := ZnEasy get: server localUrl / #echo / #foo.
		self assert: (response contentType = ZnMimeType textPlain).
		self assert: (response statusLine code = 200).
		self assert: (response entity contents includesSubstring: 'Zinc').
		self assert: (response entity contents includesSubstring: 'foo').
		self assert: (response entity contents includesSubstring: server printString) ] 
			ensure: [ server stop ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testRequestLineTooLong
	self usingClassicSocketStreamsOnWindowsDo: [
		self withServerDo: [ :server |
		self deny: server debugMode.
		self assertRequestLineTooLong: server localUrl ] ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testRespond
	self withServerDo: [ :server | | client |
		server onRequestRespond: [ :request |
			ZnRespond signalWith: (ZnResponse ok: (ZnEntity text: 'RESPONDED')).
			ZnResponse notFound: request uri ].
		client := ZnClient new.
		client get: server localUrl.
		self assert: client isSuccess.
		self assert: client response contentType equals: ZnMimeType textPlain.
		self assert: client contents equals: 'RESPONDED'.
		client close ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testSession
	self withServerDo: [ :server | | client sessionId |
		client := ZnClient new url: (server localUrl addPathSegment: #session); yourself.
		self assert: client session cookieJar cookies isEmpty.
		client get.
		self assert: client isSuccess.
		self assert: client session cookieJar cookies size = 1.
		sessionId := client session cookieJar cookies anyOne value.
		self assert: (client contents includesSubstring: sessionId).
		client get.
		self assert: client isSuccess.
		self assert: client session cookieJar cookies size = 1.
		self assert: client session cookieJar cookies anyOne value equals: sessionId.
		self assert: (client contents includesSubstring: sessionId) ]
! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testSessionExpired
	self withServerDo: [ :server | | client sessionId |
		client := ZnClient new url: (server localUrl addPathSegment: #session); yourself.
		self assert: client session cookieJar cookies isEmpty.
		client get.
		self assert: client isSuccess.
		self assert: client session cookieJar cookies size = 1.
		sessionId := client session cookieJar cookies anyOne value.
		self assert: (client contents includesSubstring: sessionId).
		"Kill the server session as if it was expired"
		server sessionManager removeSessionWithId: sessionId.
		"The client still presents the old session id but should get a new one"
		client get.
		self assert: client isSuccess.
		self assert: client session cookieJar cookies size = 1.
		self deny: client session cookieJar cookies anyOne value = sessionId ]
! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testSessionRoute
	self withServerDo: [ :server | | client sessionId |
		server route: 'r1'.
		self assert: server route equals: 'r1'.
		client := ZnClient new url: (server localUrl addPathSegment: #session); yourself.
		self assert: client session cookieJar cookies isEmpty.
		client get.
		self assert: client isSuccess.
		self assert: client session cookieJar cookies size = 1.
		sessionId := client session cookieJar cookies anyOne value.
		self assert: (client contents includesSubstring: sessionId).
		self assert: (sessionId endsWith: '.r1').
		client get.
		self assert: client isSuccess.
		self assert: client session cookieJar cookies size = 1.
		self assert: client session cookieJar cookies anyOne value equals: sessionId.
		self assert: (client contents includesSubstring: sessionId) ]
! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testSmall
	self withServerDo: [ :server | 
		| client |
		client := ZnClient new
			url: (server localUrl addPathSegment: #small);
			yourself.
		client get.
		self assert: client isSuccess.
		self assert: client response contentType equals: ZnMimeType textHtml.
		self assert: (client contents includesSubstring: 'Small').
		self assert: (client contents includesSubstring: 'This is a small HTML document').
		client close ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testTooManyHeaders
	self usingClassicSocketStreamsOnWindowsDo: [
		self withServerDo: [ :server |
			self deny: server debugMode.
			self assertTooManyHeaders: server localUrl ] ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testUrl
	self withServerDo: [ :server | 
		self assert: server url equals: server localUrl.
		server serverUrl:
					(ZnUrl new
						host: 'zn.stfx.eu';
						yourself).
		self assert: server url scheme equals: #http.
		self assert: server url host equals: 'zn.stfx.eu'.
		self assert: server url port equals: server port.
		server serverUrl: 'http://zn.stfx.eu:8080/zn'.
		self assert: server url scheme equals: #http.
		self assert: server url host equals: 'zn.stfx.eu'.
		self assert: server url port equals: 8080.
		self assert: server url path equals: 'zn' ]! !

!ZnServerTests methodsFor: 'testing' stamp: ''!
testZeroContentLength
	self withServerDo: [ :server | 
		| client |
		client := ZnClient new
			url: server localUrl / #echo;
			yourself.
		"Force a bogus zero content-length header"
		client headerAt: 'Content-Length' put: '0'.
		client get.
		self assert: client isSuccess.
		client close ]! !

!ZnServerTests methodsFor: 'private' stamp: ''!
usingClassicSocketStreamsOnWindowsDo: block
	| originalSocketStreamClass |
	originalSocketStreamClass := ZnNetworkingUtils default socketStreamClass.
	self runningOnWindows
		ifTrue: [ ZnNetworkingUtils default socketStreamClass: SocketStream ].
	block
		ensure: [ ZnNetworkingUtils default socketStreamClass: originalSocketStreamClass ]! !

!ZnServerTests methodsFor: 'private' stamp: ''!
withServerDo: block
	| server |
	server := ZnServer on: self port.
	[ 
		server start.
		self 
			assert: server isRunning & server isListening
			description: ('Failed to start server on port {1}. Is there one already?' format: { server port }).
		block cull: server 
	]
		ensure: [ server stop ]! !

TestCase subclass: #ZnStaticFileServerDelegateTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnStaticFileServerDelegateTests methodsFor: 'private' stamp: ''!
largeHtml
	^ String streamContents: [ :out |
		out << '<html><body>'.
		out << 'Les élèves Français'.
		(1 to: 4000) do: [ :each | out print: each ] separatedBy: [ out space ]. 
		out << '</html></body>' ]! !

!ZnStaticFileServerDelegateTests methodsFor: 'running' stamp: 'SvenVanCaekenberghe 11/1/2018 23:51'!
setUp
	super setUp.
	ZnFileSystemUtils
		writeStreamFor: 'small.html' 
		do: [ :stream |
			stream nextPutAll: self smallHtml ].
	ZnFileSystemUtils
		writeStreamFor: 'large.html' 
		do: [ :stream |
			stream nextPutAll: self largeHtml ].
	ZnFileSystemUtils
		writeStreamFor: 'wide.html' 
		do: [ :stream |
			stream nextPutAll: self wideHtml ].! !

!ZnStaticFileServerDelegateTests methodsFor: 'private' stamp: ''!
smallHtml
	^ '<html>
<head><title>Small</title></head>
<body><h1>Small</h1><p>This is a small HTML document</p></body>
</html>
'! !

!ZnStaticFileServerDelegateTests methodsFor: 'running' stamp: ''!
tearDown
	ZnFileSystemUtils 
		deleteIfExists: 'small.html'; 
		deleteIfExists: 'large.html'; 
		deleteIfExists: 'wide.html'.
	super tearDown	
	! !

!ZnStaticFileServerDelegateTests methodsFor: 'tests' stamp: ''!
testBasicGet
	self withServerDo: [ :server | | client |
		(client := ZnClient new) 
			beOneShot;
			url: server localUrl;
			addPath: #('local-files' 'small.html');
			get.
		self assert: client isSuccess.
		self assert: client response contentType = ZnMimeType textHtml.
		self assert: client contents equals: self smallHtml.
		self 
			assert: (ZnUtils parseHttpDate: (client response headers at: 'Modification-Date'))
			equals: (ZnFileSystemUtils modificationTimeFor: 'small.html') asUTC.
		self 
			assert: (ZnUtils parseHttpDate: (client response headers at: 'Expires')) > (DateAndTime now + 10 days).
		self 
			assert: (client response headers at: 'Cache-Control')
			equals: (server delegate maxAgeFor: ZnMimeType textHtml) ]! !

!ZnStaticFileServerDelegateTests methodsFor: 'tests' stamp: ''!
testBasicGetLarge
	self withServerDo: [ :server | | client |
		(client := ZnClient new) 
			beOneShot;
			url: server localUrl;
			addPath: #('local-files' 'large.html');
			get.
		self assert: client isSuccess.
		self assert: client response contentType = ZnMimeType textHtml.
		self assert: client contents equals: self largeHtml.
		self 
			assert: (ZnUtils parseHttpDate: (client response headers at: 'Modification-Date'))
			equals: (ZnFileSystemUtils modificationTimeFor: 'large.html') asUTC.
		self 
			assert: (ZnUtils parseHttpDate: (client response headers at: 'Expires')) > (DateAndTime now + 10 days).
		self 
			assert: (client response headers at: 'Cache-Control')
			equals: (server delegate maxAgeFor: ZnMimeType textHtml) ]! !

!ZnStaticFileServerDelegateTests methodsFor: 'tests' stamp: ''!
testBasicGetWide
	self withServerDo: [ :server | | client |
		(client := ZnClient new) 
			beOneShot;
			url: server localUrl;
			addPath: #('local-files' 'wide.html');
			get.
		self assert: client isSuccess.
		self assert: client response contentType = ZnMimeType textHtml.
		self assert: client contents equals: self wideHtml.
		self 
			assert: (ZnUtils parseHttpDate: (client response headers at: 'Modification-Date'))
			equals: (ZnFileSystemUtils modificationTimeFor: 'large.html') asUTC.
		self 
			assert: (ZnUtils parseHttpDate: (client response headers at: 'Expires')) > (DateAndTime now + 10 days).
		self 
			assert: (client response headers at: 'Cache-Control')
			equals: (server delegate maxAgeFor: ZnMimeType textHtml) ]! !

!ZnStaticFileServerDelegateTests methodsFor: 'tests' stamp: ''!
testBasicHead
	self withServerDo: [ :server | | client |
		(client := ZnClient new) 
			beOneShot;
			url: server localUrl;
			addPath: #('local-files' 'small.html');
			head.
		self assert: client isSuccess.
		self assert: client response contentType = ZnMimeType textHtml.
		self deny: client response hasEntity.
		self 
			assert: (ZnUtils parseHttpDate: (client response headers at: 'Modification-Date'))
			equals: (ZnFileSystemUtils modificationTimeFor: 'small.html') asUTC.
		self 
			assert: (ZnUtils parseHttpDate: (client response headers at: 'Expires')) > (DateAndTime now + 10 days).
		self 
			assert: (client response headers at: 'Cache-Control')
			equals: (server delegate maxAgeFor: ZnMimeType textHtml) ]! !

!ZnStaticFileServerDelegateTests methodsFor: 'tests' stamp: ''!
testIfModifiedSinceNotModified
	self withServerDo: [ :server | | client |
		(client := ZnClient new) 
			beOneShot;
			url: server localUrl;
			addPath: #('local-files' 'small.html');
			"We have to cheat a little bit since our #setUp just created the file,
			it does not alter the semantics of what we try to test"
			setIfModifiedSince: (Date today addDays: 2);
			get.
		self assert: client response isNotModified.
		self deny: client response hasEntity ]! !

!ZnStaticFileServerDelegateTests methodsFor: 'private' stamp: ''!
wideHtml
	^ '<html><h1>Czech is in Czech {1}e{2}tina.</h1></html>' format: { 269 asCharacter. 353 asCharacter}.! !

!ZnStaticFileServerDelegateTests methodsFor: 'private' stamp: ''!
withServerDo: block
	| server staticFileServerDelegate |
	server := ZnServer on: 1700 + 32 atRandom.
	[ 
		(staticFileServerDelegate := ZnStaticFileServerDelegate new)
			prefixFromString: 'local-files'; 
			directory: ZnFileSystemUtils defaultDirectory;
			mimeTypeExpirations: ZnStaticFileServerDelegate defaultMimeTypeExpirations. 
		server delegate: staticFileServerDelegate.
		server start.
		self 
			assert: server isRunning & server isListening
			description: ('Failed to start server on port {1}. Is there one already?' format: { server port }).
		block cull: server
	] 
		ensure: [ server stop ]! !

TestCase subclass: #ZnStatusLineTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnStatusLineTests methodsFor: 'testing' stamp: ''!
testInitialization
	"Make sure everything is properly initialized"
	
	ZnConstants officialHTTPStatusCodesSpec pairsDo: [ :code :reason |
		| statusLine |
		statusLine := ZnStatusLine code: code.
		self assert: statusLine code equals: code.
		self assert: statusLine reason equals: reason ]! !

!ZnStatusLineTests methodsFor: 'testing' stamp: ''!
testReading
	| statusLine string |
	string := 'HTTP/1.1 200 OK', String crlf.
	statusLine := ZnStatusLine readFrom: string readStream.
	self assert: statusLine code = 200.
	self assert: statusLine reason = 'OK'.
	self assert: statusLine version = ZnConstants defaultHTTPVersion ! !

!ZnStatusLineTests methodsFor: 'testing' stamp: ''!
testReadingNonStandardCode
	| string statusLine |
	"There exist non-standard codes out there, like those from cloudflare"
	string := 'HTTP/1.1 525 SSL Handshake Failed', String crlf.
	statusLine := ZnStatusLine readFrom: string readStream.
	self assert: statusLine code equals: 525.
	self assert: statusLine reason equals: 'SSL Handshake Failed'! !

!ZnStatusLineTests methodsFor: 'testing' stamp: ''!
testReadingWrongCode
	| string |
	string := 'HTTP/1.0 1000 OK', String crlf.
	self should: [ ZnStatusLine readFrom: string readStream ] raise: ZnUnknownHttpStatusCode ! !

!ZnStatusLineTests methodsFor: 'testing' stamp: ''!
testReadingWrongVersion
	| string |
	string := 'HTTP/0.9 200 OK', String crlf.
	self should: [ ZnStatusLine readFrom: string readStream ] raise: ZnUnknownHttpVersion! !

!ZnStatusLineTests methodsFor: 'testing' stamp: ''!
testReadingWrongVersion2
	| string |
	string := 'HTTP/0.9 Foo OK', String crlf.
	self should: [ ZnStatusLine readFrom: string readStream ] raise: ZnUnknownHttpVersion! !

!ZnStatusLineTests methodsFor: 'testing' stamp: ''!
testWriting
	| statusLine string |
	statusLine := ZnStatusLine ok.
	string := String streamContents: [ :stream | statusLine writeOn: stream ].
	self assert: string = ('HTTP/1.1 200 OK', String crlf)! !

TestCase subclass: #ZnUserAgentSessionTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnUserAgentSessionTests methodsFor: 'accessing' stamp: ''!
dummyCredential
	^ZnBasicCredential new 
		username: 'test'; 
		password: 'foobar';
		realm: 'dummy';
		domain: 'test.com';
		yourself! !

!ZnUserAgentSessionTests methodsFor: 'testing' stamp: ''!
testAccessors
	| session |
	session := ZnUserAgentSession new.
	self assert: (session cookieJar isKindOf: ZnCookieJar).
	self assert: (session credentials notNil)! !

!ZnUserAgentSessionTests methodsFor: 'testing' stamp: ''!
testAddCredential
	| session count |
	session := ZnUserAgentSession new.
	count := session credentials size.
	session addCredential: self dummyCredential.
	self assert: ((session credentials size) = (count + 1))! !

!ZnUserAgentSessionTests methodsFor: 'testing' stamp: ''!
testWithCookieJar
	self assert: ((ZnUserAgentSession withCookieJar: ZnCookieJar new) notNil)! !

TestCase subclass: #ZnUtilsTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testBase64
	| short long encoded legalCharactes |
	legalCharactes := (String withAll: ($A to: $Z), ($a to: $z), ($0 to: $9)), '='.
	short := String withAll: ((1 to: 16) collect: [ :each | 'abc' atRandom ]).
	encoded := ZnUtils encodeBase64: short.
	self assert: (ZnUtils decodeBase64: encoded) = short.
	self assert: (encoded allSatisfy: [ :each | legalCharactes includes: each ]).
	"We don't want line breaks!!"
	long := String withAll: ((1 to: 64) collect: [ :each | 'abc' atRandom ]).
	encoded := ZnUtils encodeBase64: long.
	self assert: (ZnUtils decodeBase64: encoded) = long.
	self assert: (encoded allSatisfy: [ :each | legalCharactes includes: each ])
! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testCapitalizeString
	self assert: (ZnUtils capitalizeString: 'content-type') = 'Content-Type'.
	self assert: (ZnUtils capitalizeString: 'CONTENT-type') = 'Content-Type'.
	self assert: (ZnUtils capitalizeString: 'content-TYPE') = 'Content-Type'.
	self assert: (ZnUtils capitalizeString: 'WWW-Authenticate') = 'Www-Authenticate'.
	self assert: (ZnUtils capitalizeString: 'Content-MD5') = 'Content-Md5'.! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testHttpDate
	| timestamp string |
	timestamp := (DateAndTime year: 2010 month: 9 day: 1 hour: 10 minute: 10 second: 10 offset: 0 seconds).
	string := ZnUtils httpDate: timestamp.
	self assert: (string = 'Wed, 01 Sep 2010 10:10:10 GMT')
	! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testIsCapitalizedString
	self assert: (ZnUtils isCapitalizedString: 'Content-Type').
	self assert: (ZnUtils isCapitalizedString: 'CONTENT-type') not.
	self assert: (ZnUtils isCapitalizedString: 'content-TYPE') not.
	self assert: (ZnUtils isCapitalizedString: 'WWW-Authenticate') not.
	self assert: (ZnUtils isCapitalizedString: 'Connection')! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testNextPutAll
	| data result |
	data := String new: 64 * 1024 withAll: $A.
	result := String streamContents: [ :out | ZnUtils nextPutAll: data on: out ].
	self assert: result equals: data! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testParseHttpDate
	"RFC 822, updated by RFC 1123"
	
	| timeStamp |
	timeStamp := ZnUtils parseHttpDate: 'Tue, 13 Sep 2011 08:04:49 GMT'.
	self assert: timeStamp dayOfMonth = 13.
	self assert: timeStamp monthIndex = 9.
	self assert: timeStamp year = 2011.
	self assert: timeStamp hour = 8.
	self assert: timeStamp minute = 4.
	self assert: timeStamp second = 49.
	self assert: timeStamp offset = Duration zero.! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testParseHttpDateAlternative1
	"ANSI C's asctime() format"
	
	| timeStamp |
	timeStamp := ZnUtils parseHttpDate: 'Tue Jan 01 00:00:01 2036'.
	self assert: timeStamp dayOfMonth = 1.
	self assert: timeStamp monthIndex = 1.
	self assert: timeStamp year = 2036.
	self assert: timeStamp hour = 0.
	self assert: timeStamp minute = 0.
	self assert: timeStamp second = 1.
	self assert: timeStamp offset = Duration zero! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testParseHttpDateDashes
	"RFC 850, obsoleted by RFC 1036"
	
	| timeStamp |
	timeStamp := ZnUtils parseHttpDate: 'Tuesday, 13-Sep-11 08:04:49 GMT'.
	self assert: timeStamp dayOfMonth = 13.
	self assert: timeStamp monthIndex = 9.
	self assert: timeStamp year = 2011.
	self assert: timeStamp hour = 8.
	self assert: timeStamp minute = 4.
	self assert: timeStamp second = 49.
	self assert: timeStamp offset = Duration zero! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testParseHttpDateDashesAlternative1
	"RFC 850, obsoleted by RFC 1036"
	
	| timeStamp |
	timeStamp := ZnUtils parseHttpDate: 'Tue, 13-Sep-2011 08:04:49 GMT'.
	self assert: timeStamp dayOfMonth = 13.
	self assert: timeStamp monthIndex = 9.
	self assert: timeStamp year = 2011.
	self assert: timeStamp hour = 8.
	self assert: timeStamp minute = 4.
	self assert: timeStamp second = 49.
	self assert: timeStamp offset = Duration zero! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testReadUpToEndBinary
	| data |
	data := ByteArray streamContents: [ :stream | 
		1 to: 10000 do: [ :each | 
			stream nextPut: #(1 2 3) atRandom ] ].
	self assert: (ZnUtils readUpToEnd: data readStream limit: nil) = data.
	self assert: (ZnUtils readUpToEnd: data readStream limit: 10000) = data.
	self should: [ ZnUtils readUpToEnd: data readStream limit: 9999 ] raise: ZnEntityTooLarge 
! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testReadUpToEndNonBinary
	| data |
	data := String streamContents: [ :stream | 
		1 to: 10000 do: [ :each | 
			stream nextPut: 'abc' atRandom ] ].
	self assert: (ZnUtils readUpToEnd: data readStream limit: nil) = data.
	self assert: (ZnUtils readUpToEnd: data readStream limit: 10000) = data.
	self should: [ ZnUtils readUpToEnd: data readStream limit: 9999 ] raise: ZnEntityTooLarge 
! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testStreamingBinary
	| data in out |
	data := ByteArray streamContents: [ :stream | 
		1 to: 10000 do: [ :each |
			stream nextPut: #(1 2 3) atRandom ] ].
	in := data readStream.
	out := WriteStream on: (ByteArray new: data size).
	ZnUtils streamFrom: in to: out size: data size.
	self assert: in atEnd.
	self assert: out contents = data ! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testStreamingBinaryWithoutSize
	| data in out |
	data := ByteArray streamContents: [ :stream | 
		1 to: 10000 do: [ :each |
			stream nextPut: #(1 2 3) atRandom ] ].
	in := data readStream.
	out := WriteStream on: (ByteArray new: data size).
	ZnUtils streamFrom: in to: out.
	self assert: in atEnd.
	self assert: out contents equals: data ! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testStreamingNonBinary
	| data in out |
	data := String streamContents: [ :stream | 
		1 to: 10000 do: [ :each | 
			stream nextPut: 'abc' atRandom ] ].
	in := data readStream.
	out := WriteStream on: (String new: data size).
	ZnUtils streamFrom: in to: out size: data size.
	self assert: in atEnd.
	self assert: out contents = data ! !

!ZnUtilsTests methodsFor: 'testing' stamp: ''!
testStreamingNonBinaryWithoutSize
	| data in out |
	data := String streamContents: [ :stream | 
		1 to: 10000 do: [ :each | 
			stream nextPut: 'abc' atRandom ] ].
	in := data readStream.
	out := WriteStream on: (String new: data size).
	ZnUtils streamFrom: in to: out.
	self assert: in atEnd.
	self assert: out contents equals: data ! !

Object subclass: #ZnMessageBenchmark
	instanceVariableNames: 'message representation buffer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Tests'!

!ZnMessageBenchmark commentStamp: '' prior: 0!
I am ZnMessageBenchmark helps to test the benchmarking and profiling of ZnMessage writing and reading.

Instance Variables
	buffer:					<ByteArray>
	message:				<ZnObject>
	representation:		<ByteArray>

ZnMessageBenchmark new
	simpleRequest;
	write: 10000.

ZnMessageBenchmark new
	simpleRequest;
	writeRepresentation;
	read: 10000.

ZnMessageBenchmark new
	simpleResponse;
	write: 10000.

ZnMessageBenchmark new
	simpleResponse;
	writeRepresentation;
	read: 10000.
!

!ZnMessageBenchmark class methodsFor: 'accessing' stamp: ''!
bench: messages
	| results |
	results := Dictionary new.
	messages 
		do: [ :each | | bench report |
			bench := self new.
			bench perform: each.
			bench writeRepresentation.
			report := 'Writing {1} - Reading {2}' format: { bench benchWrite. bench benchRead }.
			results at: each put: report ] 
		displayingProgress: 'Benchmarking...'.
	^ results! !

!ZnMessageBenchmark class methodsFor: 'accessing' stamp: ''!
benchAll
	^ self bench: self messages! !

!ZnMessageBenchmark class methodsFor: 'accessing' stamp: ''!
messages
	^ self requests , self responses! !

!ZnMessageBenchmark class methodsFor: 'accessing' stamp: ''!
requests
	^ #( simpleRequest standardRequest postRequest )! !

!ZnMessageBenchmark class methodsFor: 'accessing' stamp: ''!
responses
	^ #( 
		simpleResponse 
		textResponse8k textResponse64k textResponse256k
		textResponseWide8k textResponseWide64k textResponseWide256k
		asciiResponse8k asciiResponse64k asciiResponse256k
		binaryResponse8k binaryResponse64k binaryResponse256k )! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
asciiResponse256k
	message := ZnResponse ok: 
		(ZnEntity 
			with: (self randomAsciiString: 256 * 1024) 
			type: 'text/plain;charset=ascii')! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
asciiResponse64k
	message := ZnResponse ok: 
		(ZnEntity 
			with: (self randomAsciiString: 64 * 1024) 
			type: 'text/plain;charset=ascii')! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
asciiResponse8k
	message := ZnResponse ok: 
		(ZnEntity 
			with: (self randomAsciiString: 8 * 1024) 
			type: 'text/plain;charset=ascii')! !

!ZnMessageBenchmark methodsFor: 'accessing' stamp: ''!
benchRead
	^ [ self read ] bench! !

!ZnMessageBenchmark methodsFor: 'accessing' stamp: ''!
benchWrite
	^ [ self write ] bench! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
binaryResponse256k
	message := ZnResponse ok: (ZnEntity with: (self randomBytes: 256 * 1024))! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
binaryResponse64k
	message := ZnResponse ok: (ZnEntity with: (self randomBytes: 64 * 1024))! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
binaryResponse8k
	message := ZnResponse ok: (ZnEntity with: (self randomBytes: 8 * 1024))! !

!ZnMessageBenchmark methodsFor: 'accessing' stamp: ''!
buffer
	^ buffer ifNil: [  buffer := ByteArray new: ZnUtils streamingBufferSize ]! !

!ZnMessageBenchmark methodsFor: 'accessing' stamp: ''!
message
	^ message! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
postRequest
	message := (ZnRequest post: 'http://zn.stfx.eu/echo/one/two/three?param1=123&param2=foobar')
		setAcceptEncodingGzip;
		setBasicAuthenticationUsername: 'john' password: 'secret';
		entity: (ZnEntity with: (self randomBytes: 512));
		yourself! !

!ZnMessageBenchmark methodsFor: 'private' stamp: ''!
randomAsciiString: count
	^ String
		new: count
		streamContents: [ :stream | 
			count timesRepeat: [ stream nextPut: '0123456789ABCDEF' atRandom ] ]! !

!ZnMessageBenchmark methodsFor: 'private' stamp: ''!
randomBytes: count
	| bytes |
	bytes := ByteArray new: count.
	1 to: count do: [ :each | 
		bytes at: each put: (#(65 66 67 68) at: each \\ 4 + 1) ].
	^ bytes! !

!ZnMessageBenchmark methodsFor: 'private' stamp: ''!
randomUnicodeString: count
	^ String
		new: count
		streamContents: [ :stream | 
			"This is about 6% non-ASCII characters, still a ByteString"
			count timesRepeat: [ stream nextPut: '0123456789ABCDEF-élève en Français-0123456789ABCDEF' atRandom ] ]! !

!ZnMessageBenchmark methodsFor: 'private' stamp: ''!
randomUnicodeWideString: count
	| alphabet |
	"This is about 6% non-ASCII characters, 6% non-octet characters, always a WideString"
	alphabet := '0123456789ABCDEF-élève en Français-0123456789ABCDEF', 
		(WideString withAll: (#(300 400 500) collect: [ :each | each asCharacter ])).
	^ WideString
		new: count
		streamContents: [ :stream | 
			count timesRepeat: [ stream nextPut: alphabet atRandom ] ]! !

!ZnMessageBenchmark methodsFor: 'accessing' stamp: ''!
read
	| readStream |
	readStream := representation readStream.
	^ message class readFrom: readStream! !

!ZnMessageBenchmark methodsFor: 'accessing' stamp: ''!
read: count
	count timesRepeat: [ self read ]! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
simpleRequest
	message := ZnRequest get: 'http://zn.stfx.eu/dw-bench'! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
simpleResponse
	message := ZnResponse ok: (ZnEntity html: ZnDefaultServerDelegate new generateDWBench)! !

!ZnMessageBenchmark methodsFor: 'private' stamp: ''!
sizeBuffer: size
	buffer := ByteArray new: size! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
standardRequest
	message := (ZnRequest get: 'http://zn.stfx.eu/echo/one/two/three?param1=123&param2=foobar')
		setAcceptEncodingGzip;
		setBasicAuthenticationUsername: 'john' password: 'secret';
		yourself! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
textResponse256k
	message := ZnResponse ok: (ZnEntity with: (self randomUnicodeString: 256 * 1024))! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
textResponse64k
	message := ZnResponse ok: (ZnEntity with: (self randomUnicodeString: 64 * 1024))! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
textResponse8k
	message := ZnResponse ok: (ZnEntity with: (self randomUnicodeString: 8 * 1024))! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
textResponseWide256k
	message := ZnResponse ok: (ZnEntity with: (self randomUnicodeWideString: 256 * 1024))! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
textResponseWide64k
	message := ZnResponse ok: (ZnEntity with: (self randomUnicodeWideString: 64 * 1024))! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
textResponseWide8k
	message := ZnResponse ok: (ZnEntity with: (self randomUnicodeWideString: 8 * 1024))! !

!ZnMessageBenchmark methodsFor: 'accessing' stamp: ''!
write
	| writeStream |
	writeStream := self buffer writeStream.
	message writeOn: writeStream.
	^ writeStream! !

!ZnMessageBenchmark methodsFor: 'accessing' stamp: ''!
write: count
	count timesRepeat: [ self write ]! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
writeRepresentation
	representation := self write contents.
	self sizeBuffer: representation size + 1024 ! !

!ZnMessageBenchmark methodsFor: 'initialize-release' stamp: ''!
writeUsingGzipEncodingAndChunkingRepresentation
	message
		setContentEncodingGzip;
		setTransferEncodingChunked.
	representation := self write contents.
	self sizeBuffer: representation size + 1024 ! !

!ZnUrlTests methodsFor: '*zinc-tests' stamp: ''!
testRetrieveContents
	| numbers |
	numbers := 'http://zn.stfx.eu/zn/numbers.txt' asZnUrl retrieveContents.
	numbers := numbers lines collect: [ :each | each asInteger ].
	self assert: (numbers allSatisfy: [ :each | each isInteger ]).
	self should: [ 'http://zn.stfx.eu/zn/numbers-wrong.txt' asZnUrl retrieveContents ] raise: Error! !

!ZnUrlTests methodsFor: '*zinc-tests' stamp: ''!
testSaveContentsToFile
	| url target result numbers |
	url := 'http://zn.stfx.eu/zn/numbers.txt' asZnUrl.
	target := Path * 'numbers.txt'.
	ZnFileSystemUtils deleteIfExists: target .
	result := url saveContentsToFile: target.
	self assert: result exists.
	numbers := result readStreamDo: [ :in | in upToEnd ].
	numbers := numbers lines collect: [ :each | each asInteger ].
	self assert: (numbers allSatisfy: [ :each | each isInteger ]).
	ZnFileSystemUtils deleteIfExists: result ! !
